@@ -2,7 +2,7 @@
 #
 # autogen.sh - Generates the initial makefiles from a pristine CVS tree
 #
-# $Id: autogen.sh,v 1.17 2004/07/28 18:10:54 cmdjb Exp $
+# $Id: autogen.sh,v 1.20 2005/02/04 21:56:14 cmdjb Exp $
 #
 # USAGE: autogen.sh [configure options]
 #
@@ -58,6 +58,10 @@ automake=automake
 automake_vers=0
 aclocal=aclocal
 aclocal_vers=0
+autoheader=autoheader
+autoheader_vers=0
+libtoolize=libtoolize
+libtoolize_vers=0
 
 here=`pwd`
 while [ $# -ne 0 ] ; do
@@ -108,6 +112,32 @@ while [ $# -ne 0 ] ; do
       fi
     done
   fi
+  progs=`ls libtoolize* 2>/dev/null`
+  if [ "X$progs" != "X" ]; then
+    for prog in $progs; do
+      vers=`$prog --version | sed -ne '1s/^.* //p'`
+      if [ "X$vers" = "X" ]; then
+        continue
+      fi
+      if expr $vers '>' $libtoolize_vers >/dev/null; then
+        libtoolize=$prog
+        libtoolize_vers=$vers
+      fi
+    done
+  fi
+  progs=`ls autoheader* 2>/dev/null`
+  if [ "X$progs" != "X" ]; then
+    for prog in $progs; do
+      vers=`$prog --version | sed -ne '1s/^.* //p'`
+      if [ "X$vers" = "X" ]; then
+        continue
+      fi
+      if expr $vers '>' $autoheader_vers >/dev/null; then
+        autoheader=$prog
+        autoheader_vers=$vers
+      fi
+    done
+  fi
 done
 cd $here
 
@@ -118,6 +148,8 @@ set - $save_args
 echo "$program: Found autoconf $autoconf version $autoconf_vers" 1>&2
 echo "$program: Found automake $automake version $automake_vers" 1>&2
 echo "$program: Found aclocal $aclocal version $aclocal_vers" 1>&2
+echo "$program: Found autoheader $autoheader version $autoheader_vers" 1>&2
+echo "$program: Found libtoolize $libtoolize version $libtoolize_vers" 1>&2
 
 
 if ($autoconf --version) < /dev/null > /dev/null 2>&1 ; then
@@ -164,11 +196,11 @@ else
     DIE="yes"
 fi
 
-if (libtoolize --version) < /dev/null > /dev/null 2>&1 ; then
-    if (libtoolize --version | awk 'NR==1 { if( $4 >= '$libtoolize_min_vers') \
+if ($libtoolize --version) < /dev/null > /dev/null 2>&1 ; then
+    if ($libtoolize --version | awk 'NR==1 { if( $4 >= '$libtoolize_min_vers') \
 			       exit 1; exit 0; }');
     then
-       echo "$program: ERROR: \`libtoolize' is too old."
+       echo "$program: ERROR: \`$libtoolize' is too old."
        echo "           (version $libtoolize_min_vers or newer is required)"
        DIE="yes"
     fi
@@ -208,7 +240,7 @@ do
 
       # Ensure that these are created by the versions on this system
       # (indirectly via automake)
-      rm -f ltconfig ltmain.sh libtool
+      rm -f ltconfig ltmain.sh libtool stamp-h*
       # Made by automake
       rm -f missing depcomp
       # automake junk
@@ -225,15 +257,15 @@ do
 	done
       fi
 
-      echo "$program: Running libtoolize --copy --automake"
+      echo "$program: Running $libtoolize --copy --automake"
       $DRYRUN rm -f ltmain.sh libtool
-      $DRYRUN libtoolize --copy --automake
+      $DRYRUN $libtoolize --copy --automake
 
       echo "$program: Running $aclocal $aclocal_args"
       $DRYRUN $aclocal $aclocal_args
       if grep "^AM_CONFIG_HEADER" configure.ac >/dev/null; then
-	echo "$program: Running autoheader"
-	$DRYRUN autoheader
+	echo "$program: Running $autoheader"
+	$DRYRUN $autoheader
       fi
       echo "$program: Running $automake $am_opt"
       $DRYRUN $automake $automake_args $am_opt
