@@ -2,23 +2,29 @@
  *
  * ntriples_parse.c - Raptor N-Triples Parser implementation
  *
- * $Id: ntriples_parse.c,v 1.80 2004/05/18 18:24:08 cmdjb Exp $
+ * $Id: ntriples_parse.c,v 1.85 2005/01/21 05:58:02 cmdjb Exp $
  *
  * N-Triples
  * http://www.w3.org/TR/rdf-testcases/#ntriples
  *
- * Copyright (C) 2001-2004 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.bris.ac.uk/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2001-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
+ * 
  * 
  */
 
@@ -28,7 +34,7 @@
 #endif
 
 #ifdef WIN32
-#include <win32_config.h>
+#include <win32_raptor_config.h>
 #endif
 
 #include <stdio.h>
@@ -223,8 +229,7 @@ typedef enum {
 
 
 static int 
-raptor_ntriples_term_valid(raptor_parser* rdf_parser, 
-                           unsigned char c, int position, 
+raptor_ntriples_term_valid(unsigned char c, int position, 
                            raptor_ntriples_term_class term_class) 
 {
   int result=0;
@@ -255,10 +260,11 @@ raptor_ntriples_term_valid(raptor_parser* rdf_parser,
       break;
       
     case RAPTOR_TERM_CLASS_FULL:
+      result=1;
       break;
       
     default:
-      raptor_parser_fatal_error(rdf_parser, "Unknown ntriples term %d", term_class);
+      RAPTOR_FATAL2("Unknown ntriples term %d", term_class);
   }
 
   return result;
@@ -290,13 +296,13 @@ raptor_ntriples_term_valid(raptor_parser* rdf_parser,
  **/
 static int
 raptor_ntriples_term(raptor_parser* rdf_parser, 
-                     unsigned char **start, unsigned char *dest, 
+                     const unsigned char **start, unsigned char *dest, 
                      size_t *lenp, size_t *dest_lenp,
                      char end_char,
                      raptor_ntriples_term_class term_class,
                      int allow_utf8)
 {
-  unsigned char *p=*start;
+  const unsigned char *p=*start;
   unsigned char c='\0';
   size_t ulen=0;
   unsigned long unichar=0;
@@ -325,8 +331,11 @@ raptor_ntriples_term(raptor_parser* rdf_parser,
           return 1;
         }
         memcpy(dest, p-1, unichar_len);
-        unichar_len--;
+        dest+= unichar_len;
 
+        unichar_len--; /* p, *lenp were moved on by 1 earlier */
+        
+        p += unichar_len;
         (*lenp) -= unichar_len;
         rdf_parser->locator.column+= unichar_len;
         rdf_parser->locator.byte+= unichar_len;
@@ -347,7 +356,7 @@ raptor_ntriples_term(raptor_parser* rdf_parser,
         break;
       }
 
-      if(!raptor_ntriples_term_valid(rdf_parser, c, position, term_class)) {
+      if(!raptor_ntriples_term_valid(c, position, term_class)) {
         if(end_char) {
           /* end char was expected, so finding an invalid thing is an error */
           raptor_parser_error(rdf_parser, "Missing terminating '%c' (found '%c')", end_char, c);
@@ -447,10 +456,9 @@ raptor_ntriples_term(raptor_parser* rdf_parser,
 
 unsigned char*
 raptor_ntriples_string_as_utf8_string(raptor_parser* rdf_parser, 
-                                      unsigned char *src, int len,  
-                                      size_t *dest_lenp)
-{
-  unsigned char *start=src;
+                                      const unsigned char *src, int len,  
+                                      size_t *dest_lenp) {
+  const unsigned char *start=src;
   size_t length=len;
   unsigned char *dest=(unsigned char*)RAPTOR_MALLOC(cstring, len+1);
 
@@ -466,9 +474,8 @@ raptor_ntriples_string_as_utf8_string(raptor_parser* rdf_parser,
 
 
 static int
-raptor_ntriples_parse_line (raptor_parser* rdf_parser,
-                            unsigned char *buffer, size_t len) 
-{
+raptor_ntriples_parse_line(raptor_parser* rdf_parser,
+                           unsigned char *buffer, size_t len) {
   int i;
   unsigned char *p;
   unsigned char *dest;
@@ -579,7 +586,8 @@ raptor_ntriples_parse_line (raptor_parser* rdf_parser,
         rdf_parser->locator.byte++;
 
         if(raptor_ntriples_term(rdf_parser,
-                                &p, dest, &len, &term_length, 
+                                (const unsigned char**)&p, 
+                                dest, &len, &term_length, 
                                 '>', RAPTOR_TERM_CLASS_URI, 0)) {
           rc=1;
           goto cleanup;
@@ -597,7 +605,8 @@ raptor_ntriples_parse_line (raptor_parser* rdf_parser,
         rdf_parser->locator.byte++;
 
         if(raptor_ntriples_term(rdf_parser,
-                                &p, dest, &len, &term_length,
+                                (const unsigned char**)&p,
+                                dest, &len, &term_length,
                                 '"', RAPTOR_TERM_CLASS_STRING, 0)) {
           rc=1;
           goto cleanup;
@@ -622,8 +631,8 @@ raptor_ntriples_parse_line (raptor_parser* rdf_parser,
           
 
           if(raptor_ntriples_term(rdf_parser,
-                                  &p, object_literal_language,
-                                  &len, NULL,
+                                  (const unsigned char**)&p,
+                                  object_literal_language, &len, NULL,
                                   '\0', RAPTOR_TERM_CLASS_LANGUAGE, 0)) {
             rc=1;
             goto cleanup;
@@ -651,7 +660,8 @@ raptor_ntriples_parse_line (raptor_parser* rdf_parser,
           rdf_parser->locator.byte++;
 
           if(raptor_ntriples_term(rdf_parser,
-                                  &p, object_literal_datatype, &len, NULL,
+                                  (const unsigned char**)&p,
+                                  object_literal_datatype, &len, NULL,
                                   '>', RAPTOR_TERM_CLASS_URI, 0)) {
             rc=1;
             goto cleanup;
@@ -660,7 +670,7 @@ raptor_ntriples_parse_line (raptor_parser* rdf_parser,
         }
 
         if(object_literal_datatype && object_literal_language) {
-          raptor_parser_error(rdf_parser, "Typed literal used with a language - ignoring the language");
+          raptor_parser_warning(rdf_parser, "Typed literal used with a language - ignoring the language");
           object_literal_language=NULL;
         }
           
@@ -692,7 +702,8 @@ raptor_ntriples_parse_line (raptor_parser* rdf_parser,
         rdf_parser->locator.byte++;
 
         if(raptor_ntriples_term(rdf_parser,
-                                &p, dest, &len, &term_length,
+                                (const unsigned char**)&p,
+                                dest, &len, &term_length,
                                 '\0', RAPTOR_TERM_CLASS_BNODEID, 0)) {
           rc=1;
           goto cleanup;
@@ -734,7 +745,8 @@ raptor_ntriples_parse_line (raptor_parser* rdf_parser,
         rdf_parser->locator.byte++;
 
         if(raptor_ntriples_term(rdf_parser,
-                                &p, dest, &len, &term_length, 
+                                (const unsigned char**)&p,
+                                dest, &len, &term_length, 
                                 '"', RAPTOR_TERM_CLASS_STRING, 0)) {
           rc=1;
           goto cleanup;
@@ -761,8 +773,8 @@ raptor_ntriples_parse_line (raptor_parser* rdf_parser,
           }
 
           if(raptor_ntriples_term(rdf_parser,
-                                  &p, object_literal_language,
-                                  &len, NULL,
+                                  (const unsigned char**)&p,
+                                  object_literal_language, &len, NULL,
                                   '"', RAPTOR_TERM_CLASS_STRING, 0)) {
             rc=1;
             goto cleanup;
@@ -791,7 +803,8 @@ raptor_ntriples_parse_line (raptor_parser* rdf_parser,
           rdf_parser->locator.byte++;
 
           if(raptor_ntriples_term(rdf_parser,
-                                  &p, object_literal_datatype, &len, NULL,
+                                  (const unsigned char**)&p,
+                                  object_literal_datatype, &len, NULL,
                                   '>', RAPTOR_TERM_CLASS_URI, 0)) {
             rc=1;
             goto cleanup;
@@ -951,13 +964,11 @@ raptor_ntriples_parse_chunk(raptor_parser* rdf_parser,
     if(!*ptr)
       break;
 
-    if(*ptr) {
 #if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
-      RAPTOR_DEBUG3("found newline \\x%02x at offset %d\n", *ptr,
-                    ptr-line_start);
+    RAPTOR_DEBUG3("found newline \\x%02x at offset %d\n", *ptr,
+                  ptr-line_start);
 #endif
-      ntriples_parser->last_char=*ptr;
-    }
+    ntriples_parser->last_char=*ptr;
 
     len=ptr-line_start;
     rdf_parser->locator.column=0;
@@ -981,13 +992,6 @@ raptor_ntriples_parse_chunk(raptor_parser* rdf_parser,
 #endif
   }
 
-  /* exit now, no more input */
-  if(is_end) {
-    if(ptr-buffer != ntriples_parser->line_length)
-       raptor_parser_error(rdf_parser, "Junk at end of input.\"");
-    return 0;
-  }
-    
   ntriples_parser->offset=start-buffer;
 
   len=ntriples_parser->line_length - ntriples_parser->offset;
@@ -1020,6 +1024,13 @@ raptor_ntriples_parse_chunk(raptor_parser* rdf_parser,
 #endif    
   }
   
+  /* exit now, no more input */
+  if(is_end) {
+    if(ntriples_parser->offset != ntriples_parser->line_length)
+       raptor_parser_error(rdf_parser, "Junk at end of input.\"");
+    return 0;
+  }
+    
   return 0;
 }
 
