@@ -2,20 +2,26 @@
  *
  * raptor.h - Redland Parser Toolkit for RDF (Raptor) interfaces and definition
  *
- * $Id: raptor.h,v 1.120 2004/05/30 21:37:07 cmdjb Exp $
+ * $Id: raptor.h,v 1.171 2005/02/01 22:00:36 cmdjb Exp $
  *
- * Copyright (C) 2000-2004 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.bris.ac.uk/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
+ * 
  * 
  */
 
@@ -66,23 +72,34 @@ typedef void* raptor_uri;
 
 
 /* Public statics */
-extern const char * const raptor_short_copyright_string;
-extern const char * const raptor_copyright_string;
-extern const char * const raptor_version_string;
-extern const unsigned int raptor_version_major;
-extern const unsigned int raptor_version_minor;
-extern const unsigned int raptor_version_release;
-extern const unsigned int raptor_version_decimal;
+RAPTOR_API extern const char * const raptor_short_copyright_string;
+RAPTOR_API extern const char * const raptor_copyright_string;
+RAPTOR_API extern const char * const raptor_version_string;
+RAPTOR_API extern const unsigned int raptor_version_major;
+RAPTOR_API extern const unsigned int raptor_version_minor;
+RAPTOR_API extern const unsigned int raptor_version_release;
+RAPTOR_API extern const unsigned int raptor_version_decimal;
 
-extern const char * const raptor_xml_literal_datatype_uri_string;
+RAPTOR_API extern const unsigned char * const raptor_xml_namespace_uri;
+RAPTOR_API extern const unsigned char * const raptor_rdf_namespace_uri;
+RAPTOR_API extern const unsigned char * const raptor_rdf_schema_namespace_uri;
+RAPTOR_API extern const unsigned char * const raptor_xmlschema_datatypes_namespace_uri;
+RAPTOR_API extern const unsigned char * const raptor_owl_namespace_uri;
+
+RAPTOR_API extern const unsigned int raptor_rdf_namespace_uri_len;
+
+RAPTOR_API extern const unsigned char * const raptor_xml_literal_datatype_uri_string;
+RAPTOR_API extern const unsigned int raptor_xml_literal_datatype_uri_string_len;
 
 
 /* Public structure */
 typedef struct raptor_parser_s raptor_parser;
+typedef struct raptor_serializer_s raptor_serializer;
 
 typedef struct raptor_www_s raptor_www;
+typedef struct raptor_iostream_s raptor_iostream;
 
-typedef struct raptor_sax2_element_s raptor_sax2_element;
+typedef struct raptor_xml_element_s raptor_xml_element;
 typedef struct raptor_xml_writer_s raptor_xml_writer;
 
 typedef struct raptor_qname_s raptor_qname;
@@ -126,7 +143,13 @@ typedef enum {
   RAPTOR_FEATURE_NORMALIZE_LANGUAGE,
   RAPTOR_FEATURE_NON_NFC_FATAL,
   RAPTOR_FEATURE_WARN_OTHER_PARSETYPES,
-  RAPTOR_FEATURE_LAST=RAPTOR_FEATURE_WARN_OTHER_PARSETYPES
+  RAPTOR_FEATURE_CHECK_RDF_ID,
+  RAPTOR_FEATURE_RELATIVE_URIS,
+  RAPTOR_FEATURE_START_URI,
+  RAPTOR_FEATURE_WRITER_AUTO_INDENT,
+  RAPTOR_FEATURE_WRITER_AUTO_EMPTY,
+  RAPTOR_FEATURE_WRITER_INDENT_WIDTH,
+  RAPTOR_FEATURE_LAST=RAPTOR_FEATURE_WRITER_INDENT_WIDTH
 } raptor_feature;
 
 
@@ -246,19 +269,53 @@ RAPTOR_API void raptor_parse_abort(raptor_parser* rdf_parser);
 /* Utility functions */
 RAPTOR_API void raptor_print_locator(FILE *stream, raptor_locator* locator);
 RAPTOR_API int raptor_format_locator(char *buffer, size_t length, raptor_locator* locator);
- 
+RAPTOR_API int raptor_locator_line(raptor_locator *locator);
+RAPTOR_API int raptor_locator_column(raptor_locator *locator);
+RAPTOR_API int raptor_locator_byte(raptor_locator *locator);
+RAPTOR_API const char * raptor_locator_file(raptor_locator *locator);
+RAPTOR_API const char * raptor_locator_uri(raptor_locator *locator);
+
+
 RAPTOR_API const char* raptor_get_name(raptor_parser *rdf_parser);
 RAPTOR_API const char* raptor_get_label(raptor_parser *rdf_parser);
 RAPTOR_API const char* raptor_get_mime_type(raptor_parser *rdf_parser);
 
 RAPTOR_API int raptor_features_enumerate(const raptor_feature feature, const char **name, raptor_uri **uri, const char **label);
 RAPTOR_API int raptor_set_feature(raptor_parser *parser, raptor_feature feature, int value);
+RAPTOR_API int raptor_parser_set_feature_string(raptor_parser *parser, raptor_feature feature, const unsigned char *value);
 RAPTOR_API int raptor_get_feature(raptor_parser *parser, raptor_feature feature);
-RAPTOR_API raptor_feature raptor_feature_from_uri(raptor_uri *uri);
+RAPTOR_API const unsigned char* raptor_parser_get_feature_string(raptor_parser *parser, raptor_feature feature);
 RAPTOR_API void raptor_set_parser_strict(raptor_parser* rdf_parser, int is_strict);
 
+/* Get serializer names */
+RAPTOR_API int raptor_serializers_enumerate(const unsigned int counter, const char **name, const char **label, const char **mime_type, const unsigned char **uri_string);
+RAPTOR_API int raptor_serializer_syntax_name_check(const char *name);
+
+/* Serializing */
+RAPTOR_API raptor_serializer* raptor_new_serializer(const char *name);
+RAPTOR_API void raptor_free_serializer(raptor_serializer* rdf_serializer);
+
+RAPTOR_API int raptor_serialize_start(raptor_serializer *rdf_serializer, raptor_uri *uri, raptor_iostream *iostream);
+RAPTOR_API int raptor_serialize_start_to_filename(raptor_serializer *rdf_serializer, const char *filename);
+RAPTOR_API int raptor_serialize_start_to_string(raptor_serializer *rdf_serializer, raptor_uri *uri, void **string_p, size_t *length_p);
+RAPTOR_API int raptor_serialize_start_to_file_handle(raptor_serializer *rdf_serializer, raptor_uri *uri, FILE *fh);
+RAPTOR_API int raptor_serialize_set_namespace(raptor_serializer* rdf_serializer, raptor_uri *uri, const unsigned char *prefix);
+RAPTOR_API int raptor_serialize_statement(raptor_serializer* rdf_serializer, const raptor_statement *statement);
+RAPTOR_API int raptor_serialize_end(raptor_serializer *rdf_serializer);
+RAPTOR_API raptor_iostream* raptor_serializer_get_iostream(raptor_serializer *serializer);
+RAPTOR_API void raptor_serializer_set_error_handler(raptor_serializer* serializer, void *user_data, raptor_message_handler handler);
+RAPTOR_API void raptor_serializer_set_warning_handler(raptor_serializer* serializer, void *user_data, raptor_message_handler handler);
+RAPTOR_API raptor_locator* raptor_serializer_get_locator(raptor_serializer *rdf_serializer);
+RAPTOR_API int raptor_serializer_features_enumerate(const raptor_feature feature, const char **name,  raptor_uri **uri, const char **label);
+RAPTOR_API int raptor_serializer_set_feature(raptor_serializer *serializer, raptor_feature feature, int value);
+RAPTOR_API int raptor_serializer_set_feature_string(raptor_serializer *serializer, raptor_feature feature, const unsigned char *value);
+RAPTOR_API int raptor_serializer_get_feature(raptor_serializer *serializer, raptor_feature feature);
+RAPTOR_API const unsigned char *raptor_serializer_get_feature_string(raptor_serializer *serializer, raptor_feature feature);
+
 /* memory functions */
 RAPTOR_API void raptor_free_memory(void *ptr);
+RAPTOR_API void* raptor_alloc_memory(size_t size);
+RAPTOR_API void* raptor_calloc_memory(size_t nmemb, size_t size);
 
 /* URI functions */
 RAPTOR_API raptor_uri* raptor_new_uri(const unsigned char *uri_string);
@@ -284,31 +341,41 @@ RAPTOR_API void raptor_free_identifier(raptor_identifier *identifier);
 
 /* Utility functions */
 RAPTOR_API int raptor_print_ntriples_string(FILE *stream, const unsigned char *string, const char delim);
-RAPTOR_API unsigned char* raptor_ntriples_string_as_utf8_string(raptor_parser* rdf_parser, unsigned char *src, int len, size_t *dest_lenp);
-RAPTOR_API const char* raptor_ntriples_term_as_string (raptor_ntriples_term_type term);
-RAPTOR_API size_t raptor_xml_escape_string(const unsigned char *string, size_t len, unsigned char *buffer, size_t length, char quote, raptor_simple_message_handler error_handler, void *error_data);
+RAPTOR_API RAPTOR_DEPRECATED unsigned char* raptor_ntriples_string_as_utf8_string(raptor_parser* rdf_parser, const unsigned char *src, int len, size_t *dest_lenp);
+RAPTOR_API RAPTOR_DEPRECATED const char* raptor_ntriples_term_as_string(raptor_ntriples_term_type term);
+RAPTOR_API int raptor_iostream_write_string_ntriples(raptor_iostream *iostr, const unsigned char *string, size_t len, const char delim);
+RAPTOR_API void raptor_iostream_write_statement_ntriples(raptor_iostream* iostr, const raptor_statement *statement);
+RAPTOR_API int raptor_xml_escape_string(const unsigned char *string, size_t len, unsigned char *buffer, size_t length, char quote, raptor_simple_message_handler error_handler, void *error_data);
+RAPTOR_API int raptor_iostream_write_xml_escaped_string(raptor_iostream* iostr, const unsigned char *string, size_t len, char quote, raptor_simple_message_handler error_handler, void *error_data);
 
 RAPTOR_API char* raptor_vsnprintf(const char *message, va_list arguments);
 
-/* raptor_xml_writer.c */
-/* NOT PUBLIC YET - SEE raptor_internal.h */
+RAPTOR_API int raptor_xml_name_check(const unsigned char *string, size_t length, int xml_version);
+
+/* raptor_rfc2396.c */
+RAPTOR_API void raptor_uri_resolve_uri_reference (const unsigned char *base_uri, const unsigned char *reference_uri, char unsigned *buffer, size_t length);
 
 /* raptor_uri.c */
-RAPTOR_API void raptor_uri_resolve_uri_reference (const unsigned char *base_uri, const unsigned char *reference_uri, char unsigned *buffer, size_t length);
 RAPTOR_API unsigned char *raptor_uri_filename_to_uri_string(const char *filename);
 RAPTOR_API char *raptor_uri_uri_string_to_filename(const unsigned char *uri_string);
 RAPTOR_API char *raptor_uri_uri_string_to_filename_fragment(const unsigned char *uri_string, unsigned char **fragment_p);
 RAPTOR_API int raptor_uri_is_file_uri(const unsigned char* uri_string);
+RAPTOR_API unsigned char* raptor_uri_to_relative_counted_uri_string(raptor_uri *base_uri, raptor_uri *reference_uri, size_t *length_p);
+RAPTOR_API unsigned char* raptor_uri_to_relative_uri_string(raptor_uri *base_uri,  raptor_uri *reference_uri);
+RAPTOR_API void raptor_uri_print(const raptor_uri* uri, FILE *stream);
+RAPTOR_API unsigned char* raptor_uri_to_counted_string(raptor_uri *uri, size_t *len_p);
+RAPTOR_API unsigned char* raptor_uri_to_string(raptor_uri *uri);
+
 RAPTOR_API void raptor_uri_init(void);
 
 RAPTOR_API void raptor_uri_set_handler(raptor_uri_handler *handler, void *context);
 RAPTOR_API void raptor_uri_get_handler(raptor_uri_handler **handler, void **context);
 
-#define RAPTOR_RDF_MS_URI "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
-#define RAPTOR_RDF_SCHEMA_URI "http://www.w3.org/2000/01/rdf-schema#"
+#define RAPTOR_RDF_MS_URI raptor_rdf_namespace_uri
+#define RAPTOR_RDF_SCHEMA_URI raptor_rdf_schema_namespace_uri
 
-#define RAPTOR_XMLSCHEMA_DATATYPES_URI "http://www.w3.org/2001/XMLSchema#"
-#define RAPTOR_OWL_URI "http://www.w3.org/2002/07/owl#"
+#define RAPTOR_XMLSCHEMA_DATATYPES_URI raptor_xmlschema_datatypes_namespace_uri
+#define RAPTOR_OWL_URI raptor_owl_namespace_uri
 
 
 /* raptor_www */
@@ -327,6 +394,7 @@ RAPTOR_API void raptor_www_set_write_bytes_handler(raptor_www *www, raptor_www_w
 RAPTOR_API void raptor_www_set_content_type_handler(raptor_www *www, raptor_www_content_type_handler handler, void *user_data);
 RAPTOR_API void raptor_www_set_error_handler(raptor_www *www, raptor_message_handler error_handler, void *error_data);
 RAPTOR_API int raptor_www_fetch(raptor_www *www, raptor_uri *uri);
+RAPTOR_API int raptor_www_fetch_to_string(raptor_www *www, raptor_uri *uri, void **string_p, size_t *length_p, void *(*malloc_handler)(size_t size));
 RAPTOR_API void* raptor_www_get_connection(raptor_www *www);
 RAPTOR_API void raptor_www_abort(raptor_www *www, const char *reason);
 
@@ -334,13 +402,17 @@ RAPTOR_API void raptor_www_abort(raptor_www *www, const char *reason);
 /* raptor_qname - XML qnames */
 RAPTOR_API raptor_qname* raptor_new_qname(raptor_namespace_stack *nstack, const unsigned char *name, const unsigned char *value, raptor_simple_message_handler error_handler, void *error_data);
 RAPTOR_API raptor_qname* raptor_new_qname_from_namespace_local_name(raptor_namespace *ns, const unsigned char *local_name, const unsigned char *value);
+RAPTOR_API raptor_qname* raptor_qname_copy(raptor_qname *qname);
 RAPTOR_API void raptor_free_qname(raptor_qname* name);
 RAPTOR_API int raptor_qname_equal(raptor_qname *name1, raptor_qname *name2);
 /* utility function */
 RAPTOR_API raptor_uri* raptor_qname_string_to_uri(raptor_namespace_stack *nstack,  const unsigned char *name, size_t name_len, raptor_simple_message_handler error_handler, void *error_data);
+RAPTOR_API int raptor_iostream_write_qname(raptor_iostream* iostr, raptor_qname *qname);
 
 /* raptor_namespace_stack - stack of XML namespaces */
+RAPTOR_API raptor_namespace* raptor_new_namespace_from_uri(raptor_namespace_stack *nstack, const unsigned char *prefix,  raptor_uri* ns_uri, int depth);
 RAPTOR_API raptor_namespace_stack* raptor_new_namespaces(raptor_uri_handler *uri_handler, void *uri_context, raptor_simple_message_handler error_handler, void *error_data, int defaults);
+
 RAPTOR_API void raptor_namespaces_init(raptor_namespace_stack *nstack, raptor_uri_handler *handler, void *context, raptor_simple_message_handler error_handler, void *error_data, int defaults);
 RAPTOR_API void raptor_namespaces_clear(raptor_namespace_stack *nstack);
 RAPTOR_API void raptor_free_namespaces(raptor_namespace_stack *nstack);
@@ -349,6 +421,7 @@ RAPTOR_API int raptor_namespaces_start_namespace_full(raptor_namespace_stack *ns
 RAPTOR_API void raptor_namespaces_end_for_depth(raptor_namespace_stack *nstack, int depth);
 RAPTOR_API raptor_namespace* raptor_namespaces_get_default_namespace(raptor_namespace_stack *nstack);
 RAPTOR_API raptor_namespace *raptor_namespaces_find_namespace(raptor_namespace_stack *nstack, const unsigned char *prefix, int prefix_length);
+RAPTOR_API raptor_namespace* raptor_namespaces_find_namespace_by_uri(raptor_namespace_stack *nstack, raptor_uri *ns_uri);
 RAPTOR_API int raptor_namespaces_namespace_in_scope(raptor_namespace_stack *nstack, const raptor_namespace *nspace);
 
 /* raptor_namespace - XML namespace */
@@ -358,12 +431,14 @@ RAPTOR_API int raptor_namespace_copy(raptor_namespace_stack *nstack, raptor_name
 RAPTOR_API raptor_uri* raptor_namespace_get_uri(const raptor_namespace *ns);
 RAPTOR_API const unsigned char* raptor_namespace_get_prefix(const raptor_namespace *ns);
 RAPTOR_API unsigned char *raptor_namespaces_format(const raptor_namespace *ns, size_t *length_p);
+RAPTOR_API int raptor_iostream_write_namespace(raptor_iostream* iostr, raptor_namespace *ns);
+RAPTOR_API int raptor_new_namespace_parts_from_string(const unsigned char *string, unsigned char **prefix, unsigned char **uri_string);
 
 typedef struct raptor_stringbuffer_s raptor_stringbuffer;
 
 /* Sequence class */
 typedef struct raptor_sequence_s raptor_sequence;
-typedef void* (raptor_sequence_free_handler(void*));
+typedef void (raptor_sequence_free_handler(void*));
 typedef void (raptor_sequence_print_handler(void *object, FILE *fh));
 
 /* Create */
@@ -388,11 +463,16 @@ RAPTOR_API void raptor_sequence_print_string(char *data, FILE *fh);
 RAPTOR_API void raptor_sequence_print_uri(char *data, FILE *fh);
 RAPTOR_API void raptor_sequence_set_print_handler(raptor_sequence *seq, raptor_sequence_print_handler *print_handler);
 RAPTOR_API void raptor_sequence_print(raptor_sequence* seq, FILE* fh);
+RAPTOR_API int raptor_sequence_join(raptor_sequence* dest, raptor_sequence *src);
 
-/* raptor_utf8.c */
+/* Unicode and UTF8 */
 RAPTOR_API int raptor_unicode_char_to_utf8(unsigned long c, unsigned char *output);
 RAPTOR_API int raptor_utf8_to_unicode_char(unsigned long *output, const unsigned char *input, int length);
-
+RAPTOR_API int raptor_unicode_is_xml11_namestartchar(long c);
+RAPTOR_API int raptor_unicode_is_xml10_namestartchar(long c);
+RAPTOR_API int raptor_unicode_is_xml11_namechar(long c);
+RAPTOR_API int raptor_unicode_is_xml10_namechar(long c);
+RAPTOR_API int raptor_utf8_check(const unsigned char *string, size_t length);
 
 /* raptor_stringbuffer */
 RAPTOR_API raptor_stringbuffer* raptor_new_stringbuffer(void);
@@ -405,7 +485,72 @@ RAPTOR_API int raptor_stringbuffer_prepend_counted_string(raptor_stringbuffer* s
 RAPTOR_API int raptor_stringbuffer_prepend_string(raptor_stringbuffer* stringbuffer, const unsigned char *string, int do_copy);
 RAPTOR_API unsigned char * raptor_stringbuffer_as_string(raptor_stringbuffer* stringbuffer);
 RAPTOR_API size_t raptor_stringbuffer_length(raptor_stringbuffer* stringbuffer);
+RAPTOR_API int raptor_stringbuffer_copy_to_string(raptor_stringbuffer* stringbuffer, unsigned char *string, size_t length);
 
+typedef int (*raptor_iostream_init_func) (void *context);
+typedef void (*raptor_iostream_finish_func) (void *context);
+typedef int (*raptor_iostream_write_byte_func) (void *context, const int byte);
+typedef int (*raptor_iostream_write_bytes_func) (void *context, const void *ptr, size_t size, size_t nmemb);
+typedef void (*raptor_iostream_write_end_func) (void *context);
+
+typedef struct {
+  /* optional, called at most once */
+  raptor_iostream_init_func         init;
+  /* optional, called at most once */
+  raptor_iostream_finish_func       finish;
+  /* methods */
+  raptor_iostream_write_byte_func   write_byte;
+  raptor_iostream_write_bytes_func  write_bytes;
+   /* optional, called at most once */
+  raptor_iostream_write_end_func    write_end;
+} raptor_iostream_handler;
+
+
+RAPTOR_API raptor_iostream* raptor_new_iostream_from_handler(void *context, raptor_iostream_handler *handler);
+RAPTOR_API raptor_iostream* raptor_new_iostream_to_sink(void);
+RAPTOR_API raptor_iostream* raptor_new_iostream_to_filename(const char *filename);
+RAPTOR_API raptor_iostream* raptor_new_iostream_to_file_handle(FILE *handle);
+RAPTOR_API raptor_iostream* raptor_new_iostream_to_string(void **string_p, size_t *length_p, void *(*malloc_handler)(size_t size));
+RAPTOR_API void raptor_free_iostream(raptor_iostream *iostr);
+RAPTOR_API int raptor_iostream_write_bytes(raptor_iostream *iostr, const void *ptr, size_t size, size_t nmemb);
+RAPTOR_API int raptor_iostream_write_byte(raptor_iostream *iostr, const int byte);
+RAPTOR_API void raptor_iostream_write_end(raptor_iostream *iostr);
+RAPTOR_API int raptor_iostream_write_string(raptor_iostream *iostr, const void *string);
+RAPTOR_API int raptor_iostream_write_counted_string(raptor_iostream *iostr, const void *string, size_t len);
+RAPTOR_API size_t raptor_iostream_get_bytes_written_count(raptor_iostream *iostr);
+RAPTOR_API int raptor_iostream_write_decimal(raptor_iostream* iostr, int integer);
+RAPTOR_API int raptor_iostream_format_hexadecimal(raptor_iostream* iostr, unsigned int integer, int width);
+RAPTOR_API int raptor_iostream_write_stringbuffer(raptor_iostream* iostr, raptor_stringbuffer *sb);
+
+/* Parser and Serializer features */
+RAPTOR_API raptor_feature raptor_feature_from_uri(raptor_uri *uri);
+RAPTOR_API int raptor_feature_value_type(const raptor_feature feature);
+
+/* SAX2 element Class (raptor_xml_element) */
+RAPTOR_API raptor_xml_element* raptor_new_xml_element(raptor_qname* name, const unsigned char* xml_language, raptor_uri* xml_base);
+RAPTOR_API void raptor_free_xml_element(raptor_xml_element *element);
+RAPTOR_API raptor_qname* raptor_xml_element_get_name(raptor_xml_element *xml_element);
+RAPTOR_API void raptor_xml_element_set_attributes(raptor_xml_element* xml_element, raptor_qname **attributes, int count);
+RAPTOR_API void raptor_xml_element_declare_namespace(raptor_xml_element* xml_element, raptor_namespace *nspace);
+RAPTOR_API int raptor_iostream_write_xml_element(raptor_iostream *iostr, raptor_xml_element *element, raptor_namespace_stack *nstack, int is_empty, int is_end, raptor_simple_message_handler error_handler, void *error_data, int depth);
+
+/* XML Writer Class (raptor_xml_writer) */
+RAPTOR_API raptor_xml_writer* raptor_new_xml_writer(raptor_namespace_stack *nstack, raptor_uri_handler *uri_handler, void *uri_context, raptor_iostream* iostr, raptor_simple_message_handler error_handler, void *error_data, int canonicalize);
+RAPTOR_API void raptor_free_xml_writer(raptor_xml_writer* xml_writer);
+RAPTOR_API void raptor_xml_writer_empty_element(raptor_xml_writer* xml_writer, raptor_xml_element *element);
+RAPTOR_API void raptor_xml_writer_start_element(raptor_xml_writer* xml_writer, raptor_xml_element *element);
+RAPTOR_API void raptor_xml_writer_end_element(raptor_xml_writer* xml_writer, raptor_xml_element *element);
+RAPTOR_API void raptor_xml_writer_cdata(raptor_xml_writer* xml_writer, const unsigned char *str);
+RAPTOR_API void raptor_xml_writer_cdata_counted(raptor_xml_writer* xml_writer, const unsigned char *str, unsigned int length);
+RAPTOR_API void raptor_xml_writer_raw(raptor_xml_writer* xml_writer, const unsigned char *str);
+RAPTOR_API void raptor_xml_writer_raw_counted(raptor_xml_writer* xml_writer, const unsigned char *str, unsigned int length);
+RAPTOR_API void raptor_xml_writer_comment(raptor_xml_writer* xml_writer, const unsigned char *str);
+RAPTOR_API void raptor_xml_writer_comment_counted(raptor_xml_writer* xml_writer, const unsigned char *str, unsigned int length);
+RAPTOR_API int raptor_xml_writer_features_enumerate(const raptor_feature feature, const char **name,  raptor_uri **uri, const char **label);
+RAPTOR_API int raptor_xml_writer_set_feature(raptor_xml_writer *xml_writer, raptor_feature feature, int value);
+RAPTOR_API int raptor_xml_writer_set_feature_string(raptor_xml_writer *xml_writer, raptor_feature feature, const unsigned char *value);
+RAPTOR_API int raptor_xml_writer_get_feature(raptor_xml_writer *xml_writer, raptor_feature feature);
+RAPTOR_API const unsigned char *raptor_xml_writer_get_feature_string(raptor_xml_writer *xml_writer, raptor_feature feature);
 
 #ifdef __cplusplus
 }
