@@ -2,7 +2,7 @@
  *
  * raptor_libxml.c - Raptor libxml functions
  *
- * $Id: raptor_expat.c,v 1.3 2004/09/20 11:54:19 cmdjb Exp $
+ * $Id: raptor_expat.c,v 1.4 2004/11/29 22:02:56 cmdjb Exp $
  *
  * Copyright (C) 2000-2004, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -54,7 +54,7 @@
 
 
 XML_Parser
-raptor_expat_init(raptor_parser *rdf_parser) {
+raptor_expat_init(void *rdf_parser) {
   XML_Parser xp=XML_ParserCreate(NULL);
 
   /* create a new parser in the specified encoding */
