@@ -2,20 +2,26 @@
  *
  * raptor_internal.h - Redland Parser Toolkit for RDF (Raptor) internals
  *
- * $Id: raptor_internal.h,v 1.104 2004/06/06 20:54:35 cmdjb Exp $
+ * $Id: raptor_internal.h,v 1.131 2005/02/01 22:00:36 cmdjb Exp $
  *
- * Copyright (C) 2002-2004 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.bris.ac.uk/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2002-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
+ * 
  * 
  */
 
@@ -45,7 +51,8 @@ extern "C" {
 #endif
 
 #define RAPTOR_MALLOC(type, size) malloc(size)
-#define RAPTOR_CALLOC(type, size, count) calloc(size, count)
+#define RAPTOR_CALLOC(type, nmemb, size) calloc(nmemb, size)
+#define RAPTOR_REALLOC(type, ptr, size) realloc(ptr, size)
 #define RAPTOR_FREE(type, ptr)   free((void*)ptr)
 
 #ifdef RAPTOR_DEBUG
@@ -208,7 +215,7 @@ extern void raptor_set_libxml_entities(raptor_parser *rdf_parser, raptor_xml_ent
 
 #ifdef RAPTOR_XML_EXPAT
 /* raptor_expat.c exports */
-extern XML_Parser raptor_expat_init(raptor_parser *rdf_parser);
+extern XML_Parser raptor_expat_init(void *rdf_parser);
 
 /* raptor_parse.c */
 void raptor_xml_unparsed_entity_decl_handler(void *user_data, const XML_Char *entityName, const XML_Char *base, const XML_Char *systemId, const XML_Char *publicId, const XML_Char *notationName);
@@ -219,7 +226,9 @@ int raptor_xml_external_entity_ref_handler(void *user_data, const XML_Char *cont
 
 
 typedef struct raptor_parser_factory_s raptor_parser_factory;
-typedef struct raptor_set_s raptor_set;
+typedef struct raptor_serializer_factory_s raptor_serializer_factory;
+typedef struct raptor_id_set_s raptor_id_set;
+typedef struct raptor_uri_detail_s raptor_uri_detail;
 
 
 /* Raptor Namespace Stack node */
@@ -343,6 +352,11 @@ struct raptor_parser_s {
    */
   int feature_warn_other_parseTypes;
 
+  /* FEATURE:
+   * non 0 to check rdf:ID values for duplicates
+   */
+  int feature_check_rdf_id;
+
   /* stuff for our user */
   void *user_data;
 
@@ -416,6 +430,90 @@ struct raptor_parser_factory_s {
 };
 
 
+/*
+ * Raptor serializer object
+ */
+struct raptor_serializer_s {
+  /* can be filled with error location information */
+  raptor_locator locator;
+
+  /* FEATURE:
+   * non 0 to write relative URIs wherever possible
+   */
+  int feature_relative_uris;
+
+  /* FEATURE:
+   * non NULL to start serializing from this URI
+   */
+  raptor_uri* feature_start_uri;
+
+  void *error_user_data;
+  void *warning_user_data;
+
+  raptor_message_handler error_handler;
+  raptor_message_handler warning_handler;
+
+  /* non 0 if serializer had fatal error and cannot continue */
+  int failed;
+
+  /* base URI of RDF/XML */
+  raptor_uri *base_uri;
+
+  /* serializer specific stuff */
+  void *context;
+
+  /* destination stream for the serialization */
+  raptor_iostream *iostream;
+  
+  struct raptor_serializer_factory_s* factory;
+};
+
+
+/** A Serializer Factory for a syntax */
+struct raptor_serializer_factory_s {
+  struct raptor_serializer_factory_s* next;
+
+  /* syntax name */
+  const char* name;
+  /* alternate syntax name; not mentioned in enumerations */
+  const char* alias;
+
+  /* syntax readable label */
+  const char* label;
+  /* syntax MIME type (or NULL) */
+  const char* mime_type;
+  /* syntax URI (or NULL) */
+  const unsigned char* uri_string;
+  
+  /* the rest of this structure is populated by the
+     serializer-specific register function */
+  size_t context_length;
+  
+  /* create a new serializer */
+  int (*init)(raptor_serializer* serializer, const char *name);
+  
+  /* destroy a serializer */
+  void (*terminate)(raptor_serializer* serializer);
+  
+  /* add a namespace */
+  int (*declare_namespace)(raptor_serializer* serializer, raptor_uri *uri, const unsigned char *prefix);
+  
+  /* start a serialization */
+  int (*serialize_start)(raptor_serializer* serializer);
+  
+  /* serialize a statement */
+  int (*serialize_statement)(raptor_serializer* serializer, const raptor_statement *statment);
+
+  /* end a serialization */
+  int (*serialize_end)(raptor_serializer* serializer);
+  
+  /* finish the serializer factory */
+  void (*finish_factory)(raptor_serializer_factory* factory);
+};
+
+
+/* raptor_serialize.c */
+void raptor_serializer_register_factory(const char *name, const char *label, const char *mime_type, const char *alias, const unsigned char *uri_string, void (*factory) (raptor_serializer_factory*));
 
 
 /* raptor_general.c */
@@ -428,10 +526,14 @@ raptor_uri* raptor_inscope_base_uri(raptor_parser *rdf_parser);
 #ifdef RAPTOR_DEBUG
 void raptor_stats_print(raptor_parser *rdf_parser, FILE *stream);
 #endif
-
+const char* raptor_basename(const char *name);
+raptor_statement* raptor_statement_copy(const raptor_statement *statement);
+void raptor_free_statement(raptor_statement *statement);
 
 /* raptor_parse.c */
+void raptor_delete_parser_factories(void);
 
+/* raptor_general.c */
 extern void raptor_parser_fatal_error(raptor_parser* parser, const char *message, ...);
 extern void raptor_parser_error(raptor_parser* parser, const char *message, ...);
 extern void raptor_parser_simple_error(void* parser, const char *message, ...);
@@ -457,6 +559,9 @@ void raptor_xml_comment_handler(void *user_data, const unsigned char *s);
 void raptor_xml_parser_stats_print(raptor_xml_parser* rdf_xml_parser, FILE *stream);
 #endif
 
+/* raptor_feature.c */
+int raptor_features_enumerate_common(const raptor_feature feature, const char **name, raptor_uri **uri, const char **label, int flags);
+
 /* raptor_general.c */
 extern void raptor_expat_update_document_locator (raptor_parser *rdf_parser);
 extern int raptor_valid_xml_ID(raptor_parser *rdf_parser, const unsigned char *string);
@@ -482,6 +587,9 @@ extern void raptor_update_document_locator (raptor_parser *rdf_parser);
 #endif
 
 
+/* raptor_nfc.c */
+int raptor_nfc_check (const unsigned char* string, size_t len, int *error);
+
 /* raptor_nspace.c */
 
 #ifdef RAPTOR_DEBUG
@@ -523,9 +631,31 @@ void raptor_uri_init_default_handler(raptor_uri_handler *handler);
 void raptor_init_parser_rdfxml(void);
 void raptor_init_parser_ntriples(void);
 void raptor_init_parser_turtle(void);
+void raptor_init_parser_n3(void);
+
+/* raptor_rss.c */
 void raptor_init_parser_rss(void);
+void raptor_init_serializer_rss10(void);
 
+/* raptor_rfc2396.c */
+raptor_uri_detail* raptor_new_uri_detail(const unsigned char *uri_string);
+void raptor_free_uri_detail(raptor_uri_detail* uri_detail);
+unsigned char* raptor_uri_detail_to_string(raptor_uri_detail *ud, size_t* len_p);
 
+/* raptor_serializer.c */
+void raptor_init_serializer_rdfxml(void);
+void raptor_init_serializer_ntriples(void);
+void raptor_init_serializer_simple(void);
+void raptor_delete_serializer_factories(void);
+void raptor_serializer_error(raptor_serializer* serializer, const char *message, ...);
+void raptor_serializer_simple_error(void* serializer, const char *message, ...);
+void raptor_serializer_error_varargs(raptor_serializer* serializer, const char *message,  va_list arguments);
+void raptor_serializer_warning(raptor_serializer* serializer, const char *message, ...);
+void raptor_serializer_warning_varargs(raptor_serializer* serializer, const char *message, va_list arguments);
+
+/* raptor_serialize_rdfxml.c */  
+void raptor_init_serializer_rdfxmla(void);
+  
 /* raptor_utf8.c */
 int raptor_unicode_is_namestartchar(long c);
 int raptor_unicode_is_namechar(long c);
@@ -545,7 +675,7 @@ int raptor_utf8_is_nfc(const unsigned char *input, size_t length);
 #endif
 
 /* Size of buffer used in various raptor_www places for I/O  */
-#define RAPTOR_WWW_BUFFER_SIZE 256
+#define RAPTOR_WWW_BUFFER_SIZE 4096
 
 /* WWW library state */
 struct  raptor_www_s {
@@ -617,11 +747,11 @@ void raptor_www_libfetch_free(raptor_www *www);
 int raptor_www_libfetch_fetch(raptor_www *www);
 
   /* raptor_set.c */
-raptor_set* raptor_new_set(void);
-int raptor_free_set(raptor_set* set);
-int raptor_set_add(raptor_set* set, char *item, size_t item_len);
+raptor_id_set* raptor_new_id_set(void);
+void raptor_free_id_set(raptor_id_set* set);
+int raptor_id_set_add(raptor_id_set* set, raptor_uri* base_uri, const unsigned char *item, size_t item_len);
 #ifdef RAPTOR_DEBUG
-void raptor_set_stats_print(raptor_set* set, FILE *stream);
+void raptor_id_set_stats_print(raptor_id_set* set, FILE *stream);
 #endif
 
 /* raptor_sax2.c */
@@ -629,9 +759,9 @@ typedef struct raptor_sax2_s raptor_sax2;
 /*
  * SAX2 elements/attributes on stack 
  */
-struct raptor_sax2_element_s {
+struct raptor_xml_element_s {
   /* NULL at bottom of stack */
-  struct raptor_sax2_element_s *parent;
+  struct raptor_xml_element_s *parent;
   raptor_qname *name;
   raptor_qname **attributes;
   unsigned int attribute_count;
@@ -649,10 +779,14 @@ struct raptor_sax2_element_s {
   unsigned int content_cdata_seen;
   /* how many contained elements seen */
   unsigned int content_element_seen;
+
+  raptor_sequence *declared_nspaces;
 };
 
 
 struct raptor_sax2_s {
+  void* user_data;
+  
 #ifdef RAPTOR_XML_EXPAT
   XML_Parser xp;
 #ifdef EXPAT_UTF8_BOM_CRASH
@@ -683,33 +817,34 @@ struct raptor_sax2_s {
   int depth;
 
   /* stack of elements - elements add after current_element */
-  raptor_sax2_element *root_element;
-  raptor_sax2_element *current_element;
+  raptor_xml_element *root_element;
+  raptor_xml_element *current_element;
 };
 
-raptor_sax2_element* raptor_sax2_element_pop(raptor_sax2 *sax2);
-void raptor_sax2_element_push(raptor_sax2 *sax2, raptor_sax2_element* element);
-raptor_sax2_element* raptor_new_sax2_element(raptor_qname* name, const unsigned char* xml_language, raptor_uri* xml_base);
-void raptor_free_sax2_element(raptor_sax2_element *element);
+raptor_sax2* raptor_new_sax2(void *user_data);
+void raptor_free_sax2(raptor_sax2 *sax2);
+void raptor_sax2_parse_start(raptor_sax2 *sax2, raptor_uri *base_uri);
+int raptor_sax2_parse_chunk(raptor_sax2* sax2, const unsigned char *buffer, size_t len, int is_end);
+void raptor_sax2_parse_handle_errors(raptor_sax2* sax2);
+
+raptor_xml_element* raptor_xml_element_pop(raptor_sax2 *sax2);
+void raptor_xml_element_push(raptor_sax2 *sax2, raptor_xml_element* element);
+int raptor_sax2_get_depth(raptor_sax2 *sax2);
+void raptor_sax2_inc_depth(raptor_sax2 *sax2);
+void raptor_sax2_dec_depth(raptor_sax2 *sax2);
+const unsigned char* raptor_sax2_inscope_xml_language(raptor_sax2 *sax2);
+raptor_uri* raptor_sax2_inscope_base_uri(raptor_sax2 *sax2);
+
 #ifdef RAPTOR_DEBUG
-void raptor_print_sax2_element(raptor_sax2_element *element, FILE* stream);
+void raptor_print_xml_element(raptor_xml_element *element, FILE* stream);
 #endif
-unsigned char *raptor_format_sax2_element(raptor_sax2_element *element, raptor_namespace_stack* nstack, size_t *length_p, int is_end, raptor_simple_message_handler error_handler, void *error_data, int depth);
-
-/* raptor_xml_writer.c */
-/* FIXME: NOT PUBLIC YET - should be in raptor.h with RAPTOR_API added */
-raptor_xml_writer* raptor_new_xml_writer(raptor_uri_handler *uri_handler, void *uri_context, raptor_simple_message_handler error_handler, void *error_data, int canonicalize);
-void raptor_free_xml_writer(raptor_xml_writer* xml_writer);
-void raptor_xml_writer_start_element(raptor_xml_writer* xml_writer, raptor_sax2_element *element);
-void raptor_xml_writer_end_element(raptor_xml_writer* xml_writer, raptor_sax2_element *element);
-void raptor_xml_writer_cdata(raptor_xml_writer* xml_writer, const unsigned char *str, unsigned int length);
-void raptor_xml_writer_comment(raptor_xml_writer* xml_writer, const unsigned char *str, unsigned int length);
-unsigned char* raptor_xml_writer_as_string(raptor_xml_writer* xml_writer, unsigned int *length_p);
-
 
 /* turtle_parser.y and turtle_lexer.l */
 typedef struct raptor_turtle_parser_s raptor_turtle_parser;
 
+/* n3_parser.y and n3_lexer.l */
+typedef struct raptor_n3_parser_s raptor_n3_parser;
+
 typedef struct {
   raptor_identifier *subject;
   raptor_identifier *predicate;
@@ -717,6 +852,29 @@ typedef struct {
 } raptor_triple;
 
 
+/* raptor_rfc2396.c */
+struct raptor_uri_detail_s
+{
+  size_t uri_len;
+  /* buffer is the same size as the original uri_len */
+  unsigned char *buffer;
+
+  /* URI Components.  These all point into buffer */
+  unsigned char *scheme;
+  unsigned char *authority;
+  unsigned char *path;
+  unsigned char *query;
+  unsigned char *fragment;
+
+  /* Lengths of the URI Components  */
+  size_t scheme_len;
+  size_t authority_len;
+  size_t path_len;
+  size_t query_len;
+  size_t fragment_len;
+};
+  
+
 /* end of RAPTOR_INTERNAL */
 #endif
 
