@@ -2,20 +2,26 @@
  *
  * raptor_stringbuffer.c - Stringbuffer class for growing strings
  *
- * $Id: raptor_stringbuffer.c,v 1.13 2004/04/11 13:07:57 cmdjb Exp $
+ * $Id: raptor_stringbuffer.c,v 1.18 2004/10/19 22:37:33 cmdjb Exp $
  *
- * Copyright (C) 2003-2004 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.bris.ac.uk/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2003-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
+ * 
  * 
  * 
  */
@@ -26,7 +32,7 @@
 #endif
 
 #ifdef WIN32
-#include <win32_config.h>
+#include <win32_raptor_config.h>
 #endif
 
 
@@ -44,6 +50,8 @@
 #include "raptor_internal.h"
 
 
+#ifndef STANDALONE
+
 struct raptor_stringbuffer_node_s
 {
   struct raptor_stringbuffer_node_s* next;
@@ -119,7 +127,7 @@ raptor_free_stringbuffer(raptor_stringbuffer *stringbuffer)
 
 
 /**
- * raptor_stringbuffer_append_string_common: - Add a string to the stringbuffer
+ * raptor_stringbuffer_append_string_common - Add a string to the stringbuffer
  * @stringbuffer: raptor stringbuffer
  * @string: string
  * @length: length of string
@@ -177,7 +185,7 @@ raptor_stringbuffer_append_string_common(raptor_stringbuffer* stringbuffer,
 
 
 /**
- * raptor_stringbuffer_append_counted_string: - Add a string to the stringbuffer
+ * raptor_stringbuffer_append_counted_string - Add a string to the stringbuffer
  * @stringbuffer: raptor stringbuffer
  * @string: string
  * @length: length of string
@@ -199,7 +207,7 @@ raptor_stringbuffer_append_counted_string(raptor_stringbuffer* stringbuffer,
   
 
 /**
- * raptor_stringbuffer_append_string: - Add a string to the stringbuffer
+ * raptor_stringbuffer_append_string - Add a string to the stringbuffer
  * @stringbuffer: raptor stringbuffer
  * @string: string
  * @do_copy: non-0 to copy the string
@@ -219,7 +227,7 @@ raptor_stringbuffer_append_string(raptor_stringbuffer* stringbuffer,
 
 
 /**
- * raptor_stringbuffer_append_decimal: - Add an integer in decimal to the stringbuffer
+ * raptor_stringbuffer_append_decimal - Add an integer in decimal to the stringbuffer
  * @stringbuffer: raptor stringbuffer
  * @integer: integer to format as decimal and add
  * 
@@ -259,7 +267,7 @@ raptor_stringbuffer_append_decimal(raptor_stringbuffer* stringbuffer,
 
 
 /**
- * raptor_stringbuffer_append_stringbuffer: - Add a stringbuffer to the stringbuffer
+ * raptor_stringbuffer_append_stringbuffer - Add a stringbuffer to the stringbuffer
  * @stringbuffer: &raptor_stringbuffer
  * @append: &raptor_stringbuffer to append
  *
@@ -307,7 +315,7 @@ raptor_stringbuffer_append_stringbuffer(raptor_stringbuffer* stringbuffer,
 
 
 /**
- * raptor_stringbuffer_prepend_string_common: - Add a string to the start of a stringbuffer
+ * raptor_stringbuffer_prepend_string_common - Add a string to the start of a stringbuffer
  * @stringbuffer: raptor stringbuffer
  * @string: string
  * @length: length of string
@@ -364,7 +372,7 @@ raptor_stringbuffer_prepend_string_common(raptor_stringbuffer* stringbuffer,
 
 
 /**
- * raptor_stringbuffer_prepend_counted_string: - Add a string to the start of the stringbuffer
+ * raptor_stringbuffer_prepend_counted_string - Add a string to the start of the stringbuffer
  * @stringbuffer: raptor stringbuffer
  * @string: string
  * @length: length of string
@@ -386,7 +394,7 @@ raptor_stringbuffer_prepend_counted_string(raptor_stringbuffer* stringbuffer,
   
 
 /**
- * raptor_stringbuffer_prepend_string: - Add a string to the start of the stringbuffer
+ * raptor_stringbuffer_prepend_string - Add a string to the start of the stringbuffer
  * @stringbuffer: raptor stringbuffer
  * @string: string
  * @do_copy: non-0 to copy the string
@@ -406,7 +414,7 @@ raptor_stringbuffer_prepend_string(raptor_stringbuffer* stringbuffer,
 
 
 /**
- * raptor_stringbuffer_length: - Return the stringbuffer length
+ * raptor_stringbuffer_length - Return the stringbuffer length
  * @stringbuffer: raptor stringbuffer
  * 
  * Return value: size of stringbuffer
@@ -420,7 +428,7 @@ raptor_stringbuffer_length(raptor_stringbuffer* stringbuffer)
 
 
 /**
- * raptor_stringbuffer_as_string: - Return the stringbuffer as a C string
+ * raptor_stringbuffer_as_string - Return the stringbuffer as a C string
  * @stringbuffer: raptor stringbuffer
  * 
  * Note: the return value is a to a shared string that the stringbuffer
@@ -456,6 +464,47 @@ raptor_stringbuffer_as_string(raptor_stringbuffer* stringbuffer)
 }
 
 
+/**
+ * raptor_stringbuffer_copy_to_string - Copy the stringbuffer into a string
+ * @stringbuffer: raptor stringbuffer
+ * @string: output string
+ * @length: size of output string
+ * 
+ * Copies the underlying string to a pre-allocated buffer.  The
+ * output string is always '\0' terminated.
+ *
+ * Return value: non-0 on failure such as stringbuffer is empty, buffer is too small
+ **/
+int
+raptor_stringbuffer_copy_to_string(raptor_stringbuffer* stringbuffer,
+                                   unsigned char *string, size_t length)
+{
+  raptor_stringbuffer_node *node;
+  unsigned char *p;
+  
+  if(!string || length < 1)
+    return 1;
+
+  if(!stringbuffer->length)
+    return 0;
+
+  p=string;
+  for(node=stringbuffer->head; node; node=node->next) {
+    if(node->length > length) {
+      p[-1]='\0';
+      return 1;
+    }
+    strncpy((char*)p, (const char*)node->string, node->length);
+    p+= node->length;
+    length-= node->length;
+  }
+  *p='\0';
+  return 0;
+}
+
+#endif
+
+
 
 #ifdef STANDALONE
 
@@ -466,7 +515,7 @@ int main(int argc, char *argv[]);
 int
 main(int argc, char *argv[]) 
 {
-  const char *program=argv[0];
+  const char *program=raptor_basename(argv[0]);
 #define TEST_ITEMS_COUNT 9
   const char *items[TEST_ITEMS_COUNT] = { "the", "quick" ,"brown", "fox", "jumps", "over", "the", "lazy", "dog" };
   const char *items_string = "thequickbrownfoxjumpsoverthelazydog";  
@@ -483,6 +532,8 @@ main(int argc, char *argv[])
 #define TEST_APPEND_COUNT 2
   const char *test_append_results[TEST_APPEND_COUNT]={ "thebrownjumpsthedog", "quickfoxoverlazy" };
   const char *test_append_results_total="thebrownjumpsthedogquickfoxoverlazy";
+#define COPY_STRING_BUFFER_SIZE 100
+  unsigned char *copy_string;
   
 #ifdef RAPTOR_DEBUG
   fprintf(stderr, "%s: Creating string buffer\n", program);
@@ -668,6 +719,24 @@ main(int argc, char *argv[])
     exit(1);
   }
 
+
+#ifdef RAPTOR_DEBUG
+  fprintf(stderr, "%s: Copying string buffer to string\n", program);
+#endif
+
+  copy_string=(unsigned char*)malloc(COPY_STRING_BUFFER_SIZE);
+  if(raptor_stringbuffer_copy_to_string(sb1, copy_string, COPY_STRING_BUFFER_SIZE)) {
+    fprintf(stderr, "%s: copying string buffer to string failed\n",
+            program);
+    exit(1);
+  }
+  if(strcmp((const char*)copy_string, test_append_results_total)) {
+    fprintf(stderr, "%s: copied string buffer contains '%s', expected '%s'\n",
+            program, copy_string, test_append_results_total);
+    exit(1);
+  }
+  free(copy_string);
+  
   
 #ifdef RAPTOR_DEBUG
   fprintf(stderr, "%s: Freeing string buffers\n", program);
