@@ -2,9 +2,9 @@
  *
  * raptor_uri.c - Raptor URI resolving implementation
  *
- * $Id: raptor_uri.c,v 1.69 2004/09/20 11:54:22 cmdjb Exp $
+ * $Id: raptor_uri.c,v 1.82 2005/01/15 17:19:55 cmdjb Exp $
  *
- * Copyright (C) 2002-2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2002-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -167,7 +167,8 @@ raptor_default_new_uri_relative_to_base(void *context,
   raptor_uri* new_uri;
   size_t new_uri_len=strlen((const char*)base_uri)+strlen((const char*)uri_string)+1;
 
-  new_uri=(raptor_uri*)RAPTOR_MALLOC(cstring, new_uri_len+1);
+  /* +2 is for \0 plus an extra 1 for adding any missing URI path '/' */
+  new_uri=(raptor_uri*)RAPTOR_MALLOC(cstring, new_uri_len+2);
   if(!new_uri)
     return NULL;
   
@@ -219,15 +220,15 @@ static raptor_uri*
 raptor_default_new_uri_for_rdf_concept(void *context, const char *name) 
 {
   raptor_uri *new_uri;
-  char *base_uri=RAPTOR_RDF_MS_URI;
-  int base_uri_len=strlen(base_uri);
-  int new_uri_len;
+  const unsigned char *base_uri=raptor_rdf_namespace_uri;
+  unsigned int base_uri_len=raptor_rdf_namespace_uri_len;
+  unsigned int new_uri_len;
 
   new_uri_len=base_uri_len+strlen(name)+1;
   new_uri=(raptor_uri*)RAPTOR_MALLOC(cstring, new_uri_len);
   if(!new_uri)
     return NULL;
-  strcpy((char*)new_uri, base_uri);
+  strcpy((char*)new_uri, (const char*)base_uri);
   strcpy((char*)new_uri+base_uri_len, name);
   return new_uri;
 }
@@ -752,6 +753,361 @@ raptor_uri_init(void)
 }
 
 
+/**
+ * raptor_uri_path_common_base_length - Find the common base length of two URI path components
+ * @first_path: The first path (path only, not a full URI)
+ * @first_path_len: Length of first_path
+ * @second_path: The second path (path only, not a full URI)
+ * @second_path_len: Length of second_path
+ * 
+ * Return value: Length of the common base path
+ **/
+
+static int 
+raptor_uri_path_common_base_length(const unsigned char *first_path, size_t first_path_len,
+                                   const unsigned char *second_path, size_t second_path_len)
+{
+  int common_len=0;
+  const unsigned char *cur_ptr=first_path;
+  const unsigned char *prev_ptr=first_path;
+  
+  /* Compare each path component of first_path and second_path until there is
+     a mismatch. Then return the length from the start of the path to the last
+     successful match. */
+  while((cur_ptr=(const unsigned char*)memchr(cur_ptr, '/', first_path_len))) {
+    cur_ptr++;
+    if(strncmp((const char*)first_path+common_len,
+               (const char*)second_path+common_len, cur_ptr-prev_ptr))
+      break;
+
+    first_path_len -= cur_ptr - prev_ptr;
+    prev_ptr=cur_ptr;
+    common_len = prev_ptr - first_path;
+  }
+  return prev_ptr - first_path;
+}
+
+
+/**
+ * raptor_uri_path_make_relative_path - Make a relative URI path
+ * @from_path: The base path (path only, not a full URI)
+ * @from_path_len: Length of the base path
+ * @to_path: The reference path (path only, not a full URI)
+ * @to_path_len: Length of the reference path
+ * @suffix: String to be appended to the final relative path
+ * @suffix_len: Length of the suffix
+ * @result_length_p: Location to store the length of the string or NULL
+ *
+ * Return value: A newly allocated relative path string
+ **/
+
+static unsigned char *
+raptor_uri_path_make_relative_path(const unsigned char *from_path, size_t from_path_len,
+                                   const unsigned char *to_path, size_t to_path_len,
+                                   const unsigned char *suffix, size_t suffix_len,
+                                   size_t *result_length_p)
+{
+  int common_len, cur_len, final_len, up_dirs = 0, to_dir_len = 0;
+  const unsigned char *cur_ptr, *prev_ptr;
+  unsigned char *final_path, *final_path_cur;
+
+  common_len=raptor_uri_path_common_base_length(from_path, from_path_len,
+                                                to_path, to_path_len);
+  
+  if(result_length_p)
+    *result_length_p=0;
+
+  /* Count how many directories we have to go up */
+  cur_ptr = from_path + common_len;
+  prev_ptr=cur_ptr;
+  cur_len = from_path_len - common_len;
+  while((cur_ptr = (const unsigned char*)memchr(cur_ptr, '/', cur_len))) {
+    cur_ptr++;
+    up_dirs++;
+    cur_len -= cur_ptr - prev_ptr;
+    prev_ptr=cur_ptr;
+  }
+  
+  /* Calculate how many characters of to_path subdirs (counted from the 
+     common base) we have to add. */
+  cur_ptr = to_path + common_len;
+  prev_ptr=cur_ptr;
+  cur_len = to_path_len - common_len;
+  while((cur_ptr = (const unsigned char*)memchr(cur_ptr, '/', cur_len))) {
+    cur_ptr++;
+    cur_len -= cur_ptr - prev_ptr;
+    prev_ptr=cur_ptr;
+  }
+  to_dir_len = prev_ptr - (to_path + common_len);
+  
+  /* Create the final relative path */
+  final_len = up_dirs*3 + to_dir_len + suffix_len; /* 3 for each "../" */
+  final_path=(unsigned char*)RAPTOR_MALLOC(cstring, final_len + 1);
+  *final_path=0;
+  
+  /* First, add the necessary "../" parts */
+  final_path_cur=final_path;
+  while (up_dirs--) {
+    *final_path_cur++='.';
+    *final_path_cur++='.';
+    *final_path_cur++='/';
+  }
+  
+  /* Then, add the path from the common base to the to_path */
+  memcpy(final_path_cur, to_path + common_len, to_dir_len);
+  final_path_cur+=to_dir_len;
+  
+  /* Finally, add the suffix */
+  if(suffix && suffix_len) {
+    /* As a special case, if the suffix begins with a dot (".") and the final 
+       output string so far is non-empty, skip the dot. */
+    if (*suffix == '.' && final_path_cur != final_path) {
+      /* Make sure that the dot really represents a directory and it's not
+         just part of a file name like ".foo". In other words, the dot must
+         either be the only character or the next character must be the 
+         fragment or the query character. */
+      if ((suffix_len == 1) || 
+          (suffix_len > 1 && (suffix[1] == '#' || suffix[1] == '?'))) {
+        suffix++;
+        suffix_len--;
+        final_len--;
+      }
+    }
+    if(suffix_len)
+      memcpy(final_path_cur, suffix, suffix_len);
+  }
+  
+  final_path[final_len]=0;
+  
+  if (result_length_p)
+    *result_length_p=final_len;
+  
+  return final_path;
+}
+
+
+/**
+ * raptor_uri_to_relative_counted_uri_string - Get the counted relative URI string of a URI against a base URI
+ * @base_uri: The base absolute URI to resolve against
+ * @reference_uri: The reference absolute URI to use
+ * @length_p: Location to store the length of the relative URI string or NULL
+ * 
+ * Return value: A newly allocated relative URI string or NULL on failure
+ **/
+
+unsigned char*
+raptor_uri_to_relative_counted_uri_string(raptor_uri *base_uri, 
+                                          raptor_uri *reference_uri,
+                                          size_t *length_p) {
+  raptor_uri_detail *base_detail=NULL, *reference_detail;
+  const unsigned char *base, *reference_str, *base_file, *reference_file;
+  unsigned char *suffix, *cur_ptr;
+  size_t base_len, reference_len, reference_file_len, prefix_len, suffix_len;
+  unsigned char *result=NULL;
+
+  if(length_p)
+    *length_p=0;
+
+  if(!reference_uri)
+    return NULL;
+    
+  reference_str=raptor_uri_as_counted_string(reference_uri, &reference_len);
+  reference_detail=raptor_new_uri_detail(reference_str);
+  
+  if(!base_uri)
+    goto buildresult;
+  
+  base=raptor_uri_as_counted_string(base_uri, &base_len);
+  base_detail=raptor_new_uri_detail(base);
+  
+  /* The "prefix" is the scheme+authority part */
+  prefix_len = base_detail->scheme_len + base_detail->authority_len;
+  
+  /* Check if the whole URIs are equal */
+  if(raptor_uri_equals(base_uri, reference_uri)) {
+    reference_len=0;
+    goto buildresult;
+  }
+  
+  /* Check if scheme and authority of the URIs are equal */
+  if(prefix_len != (reference_detail->scheme_len + reference_detail->authority_len))
+    goto buildresult;
+  
+
+  if(!strncmp((const char*)base_detail->buffer,
+              (const char*)reference_detail->buffer, prefix_len)) {
+    
+    if(!base_detail->path)
+      goto buildresult;
+    
+    /* Find the file name components */
+    base_file = (const unsigned char*)strrchr((const char*)base_detail->path, '/');
+    if(!base_file)
+      goto buildresult;
+    base_file++;
+
+    if(!reference_detail->path)
+      goto buildresult;
+    reference_file=(const unsigned char*)strrchr((const char*)reference_detail->path, '/');
+    if(!reference_file)
+      goto buildresult;
+    reference_file++;
+    
+    reference_file_len=reference_detail->path_len -
+                       (reference_file - reference_detail->path);
+    
+    if(!strcmp((const char*)base_file, (const char*)reference_file)) {
+      /* If the file names are equal, don't put them in the relative URI */
+      reference_file=NULL;
+      reference_file_len=0;
+    } else if(*base_file && !*reference_file) {
+      /* If the base file is non-empty, but the reference file is
+       * empty, use "."  as the file name.
+       */
+      reference_file=(const unsigned char*)".";
+      reference_file_len=1;
+    }
+    
+    /* Calculate the length of the suffix (file name + query + fragment) */
+    suffix_len=reference_file_len + reference_detail->query_len + 
+               reference_detail->fragment_len;
+    
+    if (reference_detail->query)
+      suffix_len++; /* add one char for the '?' */
+    if (reference_detail->fragment)
+      suffix_len++; /* add one char for the '#' */
+    
+    /* Assemble the suffix */
+    suffix=(unsigned char*)RAPTOR_MALLOC(cstring, suffix_len + 1);
+    cur_ptr=suffix;
+    if(reference_file) {
+      memcpy(suffix, reference_file, reference_file_len);
+      cur_ptr+= reference_file_len;
+    }
+
+    if(reference_detail->query) {
+      *cur_ptr++='?';
+      memcpy(cur_ptr, reference_detail->query, reference_detail->query_len);
+      cur_ptr+= reference_detail->query_len;
+    }
+
+    if(reference_detail->fragment) {
+      *cur_ptr++='#';
+      memcpy(cur_ptr, reference_detail->fragment, reference_detail->fragment_len);
+      cur_ptr+= reference_detail->fragment_len;
+    }
+    *cur_ptr=0;
+    
+    /* Finally, create the full relative path */
+    result = raptor_uri_path_make_relative_path(base_detail->path,
+                                                base_detail->path_len,
+                                                reference_detail->path,
+                                                reference_detail->path_len,
+                                                suffix,
+                                                suffix_len,
+                                                length_p);
+    RAPTOR_FREE(cstring, suffix);
+  }
+
+  
+ buildresult:
+  /* If result is NULL at this point, it means that we were unable to find a
+     relative URI, so we'll return a full absolute URI instead. */
+  if(!result) {
+    result=(unsigned char*)RAPTOR_MALLOC(cstring, reference_len+1);
+    if(reference_len)
+      memcpy(result, reference_str, reference_len);
+    result[reference_len] = 0;
+    if(length_p)
+      *length_p=reference_len;
+  }
+  
+  if(base_detail)
+    raptor_free_uri_detail(base_detail);
+  raptor_free_uri_detail(reference_detail);
+  
+  return result;
+}
+
+
+/**
+ * raptor_uri_to_relative_uri_string - Get the relative URI string of a URI against a base URI
+ * @base_uri: The base absolute URI to resolve against
+ * @reference_uri: The reference absolute URI to use
+ * 
+ * Return value: A newly allocated relative URI string or NULL on failure
+ **/
+unsigned char*
+raptor_uri_to_relative_uri_string(raptor_uri *base_uri, 
+                                  raptor_uri *reference_uri) {
+  return raptor_uri_to_relative_counted_uri_string(base_uri,
+                                                   reference_uri, NULL);
+}
+
+
+/**
+ * raptor_uri_print - Print a URI to a file handle
+ * @uri: URI to print
+ * @stream: The file handle to print to
+ **/
+void
+raptor_uri_print(const raptor_uri* uri, FILE *stream) {
+  size_t len;
+  unsigned char *string=raptor_uri_as_counted_string((raptor_uri*)uri, &len);
+  fwrite(string, len, 1, stream);
+}
+
+
+/**
+ * raptor_uri_to_counted_string - Get a new counted string for a URI
+ * @uri: &raptor_uri object
+ * @len_p: Pointer to length (or NULL)
+ *
+ * If len_p is not NULL, the length of the string is stored in it.
+ *
+ * The memory allocated must be freed by the caller and
+ * raptor_free_memory should be used for best portability.
+ *
+ * Return value: new string or NULL on failure
+ **/
+unsigned char*
+raptor_uri_to_counted_string(raptor_uri *uri, size_t *len_p)
+{
+  size_t len;
+  unsigned char *string;
+  unsigned char *new_string;
+
+  string=raptor_uri_as_counted_string(uri, &len);
+  if(!string)
+    return NULL;
+  
+  new_string=(unsigned char*)RAPTOR_MALLOC(cstring, len+1);
+  if(!new_string)
+    return NULL;
+  
+  memcpy(new_string, uri, len+1);
+
+  if(len_p)
+    *len_p=len;
+  return new_string;
+}
+
+
+/**
+ * raptor_uri_to_string - Get a new string for a URI
+ * @uri: &raptor_uri object
+ * @len_p: Pointer to length (or NULL)
+ *
+ * The memory allocated must be freed by the caller and
+ * raptor_free_memory should be used for best portability.
+ *
+ * Return value: new string or NULL on failure
+ **/
+unsigned char*
+raptor_uri_to_string(raptor_uri *uri)
+{
+  return raptor_uri_to_counted_string(uri, NULL);
+}
 
 
 #ifdef STANDALONE
@@ -816,6 +1172,35 @@ assert_uri_to_filename (const char *uri, const char *reference_filename)
 }
 
 
+static int
+assert_uri_to_relative(const char *base, const char *uri, const char *relative)
+{
+  unsigned char *output;
+  int result;
+  raptor_uri* base_uri=NULL;
+  raptor_uri* reference_uri=raptor_new_uri((const unsigned char*)uri);
+  size_t length=0;
+
+  if(base)
+    base_uri=raptor_new_uri((const unsigned char*)base);
+  
+  output=raptor_uri_to_relative_counted_uri_string(base_uri, reference_uri, 
+                                                   &length);
+  result=strcmp(relative, (const char*)output);
+  if (result) {
+    fprintf(stderr,
+            "%s: raptor_uri_string_to_relative_uri_string FAILED: base='%s', uri='%s', expected='%s', got='%s'\n",
+            program, base, uri, relative, output);
+    RAPTOR_FREE(cstring, output);
+    return 1;
+  }
+  RAPTOR_FREE(cstring, output);
+  raptor_free_uri(base_uri);
+  raptor_free_uri(reference_uri);
+  return 0;
+}
+
+
 int
 main(int argc, char *argv[]) 
 {
@@ -934,6 +1319,24 @@ main(int argc, char *argv[])
   raptor_free_uri(uri3);
   raptor_free_uri(uri2);
   raptor_free_uri(uri1);
+  
+  failures += assert_uri_to_relative(NULL, "http://example.com/foo/bar", "http://example.com/foo/bar");
+  failures += assert_uri_to_relative("", "http://example.com/foo/bar", "http://example.com/foo/bar");
+  failures += assert_uri_to_relative("foo:", "http://example.com/foo/bar", "http://example.com/foo/bar");
+  failures += assert_uri_to_relative("http://example.com/base/foo?foo#foo", "http://example.com/base/bar?bar#bar", "bar?bar#bar");
+  failures += assert_uri_to_relative("http://example.com/base/foo", "http://example.com/base/foo/", "foo/");
+  failures += assert_uri_to_relative("http://example.com/base/foo", "http://example.com/base/foo/.foo", "foo/.foo");
+  failures += assert_uri_to_relative("http://example.com/base/foo", "http://example.com/base/foo/.foo#bar", "foo/.foo#bar");
+  failures += assert_uri_to_relative("http://example.com/base/foo", "http://example.com/base/foo/bar", "foo/bar");
+  failures += assert_uri_to_relative("http://example.com/base/foo", "http://example.com/base/foo#bar", "#bar");
+  failures += assert_uri_to_relative("http://example.com/base/foo", "http://example.com/base/bar#foo", "bar#foo");
+  failures += assert_uri_to_relative("http://example.com/base/foo", "http://example.com/otherbase/bar", "../otherbase/bar");
+  failures += assert_uri_to_relative("http://example.com/base/foo", "http://example.com/base/#foo", ".#foo");
+  failures += assert_uri_to_relative("http://example.com/base/foo", "http://example2.com/base/bar", "http://example2.com/base/bar");
+  failures += assert_uri_to_relative("http://example.com/base/one?path=/should/be/ignored", "http://example.com/base/two?path=/should/be/ignored", "two?path=/should/be/ignored");
+  failures += assert_uri_to_relative("http://example.org/base#", "http://www.foo.org", "http://www.foo.org");
+  failures += assert_uri_to_relative("http://example.org", "http://a.example.org/", "http://a.example.org/");
+  failures += assert_uri_to_relative("http://example.org", "http://a.example.org", "http://a.example.org");
 
   return failures ;
 }
