@@ -2,20 +2,26 @@
  *
  * raptor_utf8.c - Raptor UTF-8 and Unicode support
  *
- * $Id: raptor_utf8.c,v 1.11 2004/05/19 20:31:53 cmdjb Exp $
+ * $Id: raptor_utf8.c,v 1.19 2004/12/25 12:02:32 cmdjb Exp $
  *
- * Copyright (C) 2002 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.org/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2002-2004, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
+ * 
  * 
  */
 
@@ -25,7 +31,7 @@
 #endif
 
 #ifdef WIN32
-#include <win32_config.h>
+#include <win32_raptor_config.h>
 #endif
 
 #include <stdio.h>
@@ -34,15 +40,12 @@
 #include <stdlib.h>
 #endif
 
-#ifdef RAPTOR_NFC_CHECK
-#ifdef HAVE_G_UTF8_NORMALIZE
-#include <glib.h>
-#endif
-#endif
-
 /* Raptor includes */
 #include "raptor.h"
 #include "raptor_internal.h"
+#ifdef RAPTOR_NFC_CHECK
+#include "raptor_nfc.h"
+#endif
 
 
 /*
@@ -191,37 +194,25 @@ raptor_utf8_to_unicode_char(unsigned long *output,
 }
 
 
-#undef RAPTOR_XML_1_1
-
-#ifndef RAPTOR_XML_1_1  
 static int raptor_unicode_is_letter(long c);
 static int raptor_unicode_is_basechar(long c);
 static int raptor_unicode_is_ideographic(long c);
 static int raptor_unicode_is_combiningchar(long c);
 static int raptor_unicode_is_digit(long c);
 static int raptor_unicode_is_extender(long c);
-#endif
 
 
-/**
- * raptor_unicode_is_namestartchar - Check if Unicode character is legal to start an XML Namespace Name 
- * @c: Unicode character to check
- * 
- * Return value: non-0 if the character is legal
- **/
 int
-raptor_unicode_is_namestartchar(long c) 
-{
-#ifdef RAPTOR_XML_1_1
-  /* Namespaces in XML 1.1 PR
-   *   http://www.w3.org/TR/2003/PR-xml-names11-20031105/#NT-NCNameStartChar
+raptor_unicode_is_xml11_namestartchar(long c) {
+  /* Namespaces in XML 1.1 REC 2004-02-04
+   *   http://www.w3.org/TR/2004/REC-xml11-20040204/#NT-NameStartChar
    * updating
-   *   Extensible Markup Language (XML) 1.1 PR
-   *   http://www.w3.org/TR/2003/PR-xml11-20031105/#NT-NameStartChar
+   *   Extensible Markup Language (XML) 1.1 REC 2004-02-04
+   *   http://www.w3.org/TR/2004/REC-xml11-20040204/ sec 2.3, [4a]
    * excluding the ':'
    */
   return (((c >= 0x0041)  && (c <= 0x005A)) || /* [A-Z] */
-          (c == 0x005F) ||                    /* '_' */
+          (c == 0x005F) ||                     /* '_' */
           ((c >= 0x0061)  && (c <= 0x007A)) || /* [a-z] */
           ((c >= 0x00C0)  && (c <= 0x00D6)) ||
           ((c >= 0x00D8)  && (c <= 0x00F6)) ||
@@ -235,18 +226,75 @@ raptor_unicode_is_namestartchar(long c)
           ((c >= 0xF900)  && (c <= 0xFDCF)) ||
           ((c >= 0xFDF0)  && (c <= 0xFFFD)) ||
           ((c >= 0x10000) && (c <= 0xEFFFF)));
-#else
-  /* Namespaces in XML REC
+}
+
+
+int
+raptor_unicode_is_xml10_namestartchar(long c) {
+  /* Namespaces in XML REC 1999-01-14
    *   http://www.w3.org/TR/1999/REC-xml-names-19990114/#NT-NCName
    * updating
-   *   Extensible Markup Language (XML) 1.0 (Second Edition) REC
-   *   http://www.w3.org/TR/2000/REC-xml-20001006#NT-Name
+   *   Extensible Markup Language (XML) 1.0 (Third Edition) REC 2004-02-04
+   *   http://www.w3.org/TR/2004/REC-xml-20040204/
    * excluding the ':'
    */
   return (raptor_unicode_is_letter(c) ||
           (c == '_'));
+}
+
+
+/**
+ * raptor_unicode_is_namestartchar - Check if Unicode character is legal to start an XML Namespace Name 
+ * @c: Unicode character to check
+ * 
+ * Return value: non-0 if the character is legal
+ **/
+int
+raptor_unicode_is_namestartchar(long c) {
+#ifdef RAPTOR_XML_1_1
+   return raptor_unicode_is_xml11_namestartchar(c);
+#else
+   return raptor_unicode_is_xml10_namestartchar(c);
 #endif
 }
+
+
+int
+raptor_unicode_is_xml11_namechar(long c) {
+  /* Namespaces in XML 1.1 REC 2004-02-04
+   *   http://www.w3.org/TR/2004/REC-xml11-20040204/
+   * updating
+   *   Extensible Markup Language (XML) 1.1 REC 2004-02-04
+   *   http://www.w3.org/TR/2004/REC-xml11-20040204/ sec 2.3, [4a]
+   * excluding the ':'
+   */
+  return (raptor_unicode_is_namestartchar(c) ||
+          (c == 0x002D) || /* '-' */
+          (c == 0x002E) || /* '.' */
+          (c >= 0x0030 && c <= 0x0039) || /* 0-9 */
+          (c == 0x00B7) ||
+          (c >= 0x0300 && c <=0x036F) ||
+          (c >= 0x203F && c <=0x2040));
+}
+
+
+int
+raptor_unicode_is_xml10_namechar(long c) {
+  /* Namespaces in XML REC 1999-01-14
+   *   http://www.w3.org/TR/1999/REC-xml-names-19990114/#NT-NCNameChar
+   * updating
+   *   Extensible Markup Language (XML) 1.0 (Third Edition) REC 2004-02-04
+   *   http://www.w3.org/TR/2004/REC-xml-20040204/
+   * excluding the ':'
+   */
+  return (raptor_unicode_is_letter(c) ||
+          raptor_unicode_is_digit(c) ||
+          (c == 0x002E) || /* '.' */
+          (c == 0x002D) || /* '-' */
+          (c == 0x005F) || /* '_' */
+          raptor_unicode_is_combiningchar(c) ||
+          raptor_unicode_is_extender(c));
+}
  
 
 /**
@@ -259,40 +307,12 @@ int
 raptor_unicode_is_namechar(long c) 
 {
 #ifdef RAPTOR_XML_1_1
-  /* Namespaces in XML 1.1 PR
-   *   http://www.w3.org/TR/2003/PR-xml11-20031105/#NT-NameChar
-   * updating
-   *   Extensible Markup Language (XML) 1.1 WD
-   *   http://www.w3.org/TR/2002/CR-xml11-20021015/ sec 2.3, [4a]
-   * excluding the ':'
-   */
-  return (raptor_unicode_is_namestartchar(c) ||
-          (c == 0x005F) || /* '_' */
-          (c == 0x002E) || /* '.' */
-          (c >= 0x0030 && c <= 0x0039) || /* 0-9 */
-          (c == 0x00B7) ||
-          (c >= 0x0300 && c <=0x036F) ||
-          (c >= 0x203F && c <=0x2040));
+   return raptor_unicode_is_xml11_namechar(c);
 #else
-  /* Namespaces in XML REC
-   *   http://www.w3.org/TR/1999/REC-xml-names-19990114/#NT-NCNameChar
-   * updating
-   *   Extensible Markup Language (XML) 1.0 (Second Edition) REC
-   *   http://www.w3.org/TR/2000/REC-xml-20001006#NT-NameChar
-   * excluding the ':'
-   */
-  return (raptor_unicode_is_letter(c) ||
-          raptor_unicode_is_digit(c) ||
-          (c == 0x002E) || /* '.' */
-          (c == 0x002D) || /* '-' */
-          (c == 0x005F) || /* '_' */
-          raptor_unicode_is_combiningchar(c) ||
-          raptor_unicode_is_extender(c));
+   return raptor_unicode_is_xml10_namechar(c);
 #endif
 }
- 
 
-#ifndef RAPTOR_XML_1_1  
 
 /*
  * All this below was derived by machine-transforming the classes in Appendix B
@@ -667,9 +687,6 @@ raptor_unicode_is_extender(long c)
          (c >= 0x309D && c <= 0x309E ) ||
          (c >= 0x30FC && c <= 0x30FE ));
 }
-         
-
-#endif /* if not XML 1.1 */
 
 
 /**
@@ -684,9 +701,6 @@ raptor_utf8_is_nfc(const unsigned char *input, size_t length)
 {
   unsigned int i;
   int plain=1;
-#if defined(RAPTOR_NFC_CHECK) && defined (HAVE_G_UTF8_NORMALIZE)
-  unsigned char *norm;
-#endif
   
   for(i=0; i<length; i++)
     if(input[i]>0x7f) {
@@ -696,17 +710,37 @@ raptor_utf8_is_nfc(const unsigned char *input, size_t length)
     
   if(plain)
     return 1;
-  
-#if defined(RAPTOR_NFC_CHECK) && defined(HAVE_G_UTF8_NORMALIZE)
-  norm=(unsigned char*)g_utf8_normalize((const gchar*)input, length, G_NORMALIZE_NFKC);
-  for(i=0; i<length; i++)
-    if(input[i] != norm[i]) {
-      SYSTEM_FREE(norm);
-      return 0;
-    }
-  
-  SYSTEM_FREE(norm);
-#endif
 
+#ifdef RAPTOR_NFC_CHECK  
+  return raptor_nfc_check(input, length, NULL);
+#else
+  return 1;
+#endif
+}
+
+
+/**
+ * raptor_utf8_check - Check a string is UTF-8
+ * @string: UTF-8 string
+ * @length: length of string
+ * 
+ * Return value: Non 0 if the string is UTF-8
+ **/
+int
+raptor_utf8_check(const unsigned char *string, size_t length)
+{
+  while(length > 0) {
+    unsigned long unichar=0;
+
+    int unichar_len=raptor_utf8_to_unicode_char(&unichar, string, length);
+    if(unichar_len < 0 || unichar_len > (int)length)
+      return 0;
+
+    if(unichar < 0 || unichar > 0x10ffff)
+      return 0;
+  
+    string += unichar_len;
+    length -= unichar_len;
+  }
   return 1;
 }
