@@ -2,20 +2,26 @@
  *
  * rdfdump.c - Raptor RDF Parser example code 
  *
- * $Id: rdfdump.c,v 1.92 2004/06/08 16:52:19 cmdjb Exp $
+ * $Id: rdfdump.c,v 1.106 2005/02/01 22:51:30 cmdjb Exp $
  *
- * Copyright (C) 2000-2004 David Beckett - http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology - http://www.ilrt.bris.ac.uk/
- * University of Bristol - http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
+ * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
+ * University of Bristol, UK http://www.bristol.ac.uk/
  * 
- * This package is Free Software or Open Source available under the
- * following licenses (these are alternatives):
- *   1. GNU Lesser General Public License (LGPL)
- *   2. GNU General Public License (GPL)
- *   3. Mozilla Public License (MPL)
+ * This package is Free Software and part of Redland http://librdf.org/
+ * 
+ * It is licensed under the following three licenses as alternatives:
+ *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+ *   2. GNU General Public License (GPL) V2 or any newer version
+ *   3. Apache License, V2.0 or any newer version
+ * 
+ * You may not use this file except in compliance with at least one of
+ * the above three licenses.
  * 
  * See LICENSE.html or LICENSE.txt at the top of this package for the
- * full license terms.
+ * complete terms and further detail along with the license texts for
+ * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
+ * 
  * 
  */
 
@@ -25,7 +31,7 @@
 #endif
 
 #ifdef WIN32
-#include <win32_config.h>
+#include <win32_raptor_config.h>
 #endif
 
 #include <stdio.h>
@@ -73,7 +79,7 @@ static int count=0;
 
 static int statement_count=0;
 
-static enum { OUTPUT_FORMAT_SIMPLE, OUTPUT_FORMAT_NTRIPLES } output_format = OUTPUT_FORMAT_SIMPLE;
+static raptor_serializer* serializer=NULL;
 
 
 static
@@ -82,9 +88,6 @@ void print_statements(void *user_data, const raptor_statement *statement)
   statement_count++;
   if(count)
     return;
-  
-  if(output_format == OUTPUT_FORMAT_SIMPLE)
-    fprintf(stdout, "%s: Statement: ", program);
 
   /* replace newlines with spaces if object is a literal string */
   if(replace_newlines && 
@@ -95,11 +98,8 @@ void print_statements(void *user_data, const raptor_statement *statement)
         *s=' ';
   }
 
-  if(output_format == OUTPUT_FORMAT_SIMPLE)
-    raptor_print_statement(statement, stdout);
-  else
-    raptor_print_statement_as_ntriples(statement, stdout);
-  fputc('\n', stdout);
+  raptor_serialize_statement(serializer, statement);
+  return;
 }
 
 
@@ -177,6 +177,23 @@ rdfdump_warning_handler(void *data, raptor_locator *locator,
   warning_count++;
 }
 
+struct namespace_decl
+{
+  unsigned char *prefix;
+  unsigned char *uri_string;
+};
+
+
+static void
+rdfdump_free_namespace_decl(void* data) {
+  struct namespace_decl* nsd=(struct namespace_decl*)data;
+  if(nsd->prefix)
+    raptor_free_memory(nsd->prefix);
+  if(nsd->uri_string)
+    raptor_free_memory(nsd->uri_string);
+  raptor_free_memory(nsd);
+}
+
 
 int
 main(int argc, char *argv[]) 
@@ -188,6 +205,7 @@ main(int argc, char *argv[])
   int rc;
   int scanning=0;
   const char *syntax_name="rdfxml";
+  const char *serializer_syntax_name="ntriples";
   int strict_mode=0;
   int usage=0;
   int help=0;
@@ -196,8 +214,13 @@ main(int argc, char *argv[])
   raptor_uri *uri;
   char *p;
   char *filename=NULL;
-  raptor_feature feature;
-  int feature_value= -1;
+  raptor_feature parser_feature=(raptor_feature)-1;
+  int parser_feature_value= -1;
+  unsigned char* parser_feature_string_value=NULL;
+  raptor_feature serializer_feature=(raptor_feature)-1;
+  int serializer_feature_value= -1;
+  unsigned char* serializer_feature_string_value=NULL;
+  raptor_sequence *namespace_declarations=NULL;
 
   program=argv[0];
   if((p=strrchr(program, '/')))
@@ -207,7 +230,7 @@ main(int argc, char *argv[])
   argv[0]=program;
 
   raptor_init();
-  
+
   while (!usage && !help)
   {
     int c;
@@ -239,38 +262,99 @@ main(int argc, char *argv[])
           if(!strcmp(optarg, "help")) {
             int i;
             
-            fprintf(stderr, "%s: Valid features for `" HELP_ARG(f, feature) "' are:\n", program);
-            for(i=0; 1; i++) {
+            fprintf(stderr, "%s: Valid parser features are:\n", program);
+            for(i=0; i <= RAPTOR_FEATURE_LAST; i++) {
               const char *feature_name;
               const char *feature_label;
-              if(raptor_features_enumerate((raptor_feature)i, &feature_name, NULL, &feature_label))
-                break;
-              printf("  %-20s  %s\n", feature_name, feature_label);
+              if(!raptor_features_enumerate((raptor_feature)i, &feature_name, NULL, &feature_label)) {
+                const char *feature_type=(raptor_feature_value_type((raptor_feature)i) == 0) ? "" : " (string)";
+                printf("  %-20s  %s%s\n", feature_name, feature_label, 
+                       feature_type);
+              }
             }
-            fputs("\nFeature values are decimal integers 0 or larger, defaulting to 1 if omitted.\n", stderr);
+            fprintf(stderr, "%s: Valid serializer features are:\n", program);
+            for(i=0; i <= RAPTOR_FEATURE_LAST; i++) {
+              const char *feature_name;
+              const char *feature_label;
+              if(!raptor_serializer_features_enumerate((raptor_feature)i, &feature_name, NULL, &feature_label)) {
+                const char *feature_type=(raptor_feature_value_type((raptor_feature)i) == 0) ? "" : " (string)";
+                printf("  %-20s  %s%s\n", feature_name, feature_label, 
+                       feature_type);
+              }
+            }
+            fputs("Features are set with `" HELP_ARG(f, feature) " FEATURE=VALUE or `-f FEATURE'\nand take a decimal integer VALUE except where noted, defaulting to 1 if omitted.\n", stderr);
+            fputs("\nA feature of the form xmlns:PREFIX=\"URI\" can be used to declare output\nnamespace prefixes and names for serializing using an XML-style syntax\nEither or both of PREFIX or URI can be omitted such as -f xmlns=\"URI\"\nThis form can be repeated for multiple declarations.\n", stderr);
             exit(0);
+          } else if(!strncmp(optarg, "xmlns", 5)) {
+            struct namespace_decl *nd;
+            nd=(struct namespace_decl *)raptor_alloc_memory(sizeof(struct namespace_decl));
+            if(raptor_new_namespace_parts_from_string((unsigned char*)optarg,
+                                                      &nd->prefix,
+                                                      &nd->uri_string)) {
+              fprintf(stderr, "%s: Bad xmlns syntax in '%s'\n", program, 
+                      optarg);
+              rdfdump_free_namespace_decl(nd);
+              exit(0);
+            }
+
+            if(!namespace_declarations)
+              namespace_declarations=raptor_new_sequence(rdfdump_free_namespace_decl, NULL);
+
+            raptor_sequence_push(namespace_declarations, nd);
           } else {
             int i;
             size_t arg_len=strlen(optarg);
             
-            for(i=0; 1; i++) {
+            for(i=0; i <= RAPTOR_FEATURE_LAST; i++) {
               const char *feature_name;
               size_t len;
               
               if(raptor_features_enumerate((raptor_feature)i, &feature_name, NULL, NULL))
-                break;
+                continue;
               len=strlen(feature_name);
               if(!strncmp(optarg, feature_name, len)) {
-                feature=(raptor_feature)i;
-                if(len < arg_len && optarg[len] == '=')
-                  feature_value=atoi(&optarg[len+1]);
-                else if(len == arg_len)
-                  feature_value=1;
+                parser_feature=(raptor_feature)i;
+                if(raptor_feature_value_type(parser_feature) == 0) {
+                  if(len < arg_len && optarg[len] == '=')
+                    parser_feature_value=atoi(&optarg[len+1]);
+                  else if(len == arg_len)
+                    parser_feature_value=1;
+                } else {
+                  if(len < arg_len && optarg[len] == '=')
+                    parser_feature_string_value=(unsigned char*)&optarg[len+1];
+                  else if(len == arg_len)
+                    parser_feature_string_value=(unsigned char*)"";
+                }
                 break;
               }
             }
             
-            if(feature_value < 0 ) {
+            for(i=0; i <= RAPTOR_FEATURE_LAST; i++) {
+              const char *feature_name;
+              size_t len;
+              
+              if(raptor_serializer_features_enumerate((raptor_feature)i, &feature_name, NULL, NULL))
+                continue;
+              len=strlen(feature_name);
+              if(!strncmp(optarg, feature_name, len)) {
+                serializer_feature=(raptor_feature)i;
+                if(raptor_feature_value_type(serializer_feature) == 0) {
+                  if(len < arg_len && optarg[len] == '=')
+                    serializer_feature_value=atoi(&optarg[len+1]);
+                  else if(len == arg_len)
+                    serializer_feature_value=1;
+                } else {
+                  if(len < arg_len && optarg[len] == '=')
+                    serializer_feature_string_value=(unsigned char*)&optarg[len+1];
+                  else if(len == arg_len)
+                    serializer_feature_string_value=(unsigned char*)"";
+                }
+                break;
+              }
+            }
+            
+            if(parser_feature_value < 0 && !parser_feature_string_value &&
+               serializer_feature_value < 0 && !serializer_feature_string_value) {
               fprintf(stderr, "%s: invalid argument `%s' for `" HELP_ARG(f, feature) "'\nTry '%s " HELP_ARG(f, feature) " help' for a list of valid features\n",
                       program, optarg, program);
               usage=1;
@@ -320,15 +404,24 @@ main(int argc, char *argv[])
 
       case 'o':
         if(optarg) {
-          if(!strcmp(optarg, "simple"))
-            output_format=OUTPUT_FORMAT_SIMPLE;
-          else if (!strcmp(optarg, "ntriples"))
-            output_format=OUTPUT_FORMAT_NTRIPLES;
+          if(raptor_serializer_syntax_name_check(optarg))
+            serializer_syntax_name=optarg;
           else {
+            int i;
+            
             fprintf(stderr, "%s: invalid argument `%s' for `" HELP_ARG(o, output) "'\n",
                     program, optarg);
-            fprintf(stderr, "Valid arguments are:\n  `simple'   for a simple format (default)\n  `ntriples' for N-Triples\n");
+            fprintf(stderr, "Valid arguments are:\n");
+            for(i=0; 1; i++) {
+              const char *help_name;
+              const char *help_label;
+              if(raptor_serializers_enumerate(i, &help_name, &help_label, NULL, NULL))
+                break;
+              printf("  %-12s for %s\n", help_name, help_label);
+            }
             usage=1;
+            break;
+            
           }
         }
         break;
@@ -398,33 +491,43 @@ main(int argc, char *argv[])
     puts("Parse RDF content at the source URI into RDF triples.");
     puts("\nMain options:");
     puts(HELP_TEXT("h", "help            ", "Print this help, then exit"));
-    puts(HELP_TEXT("i FORMAT", "input FORMAT ", "Set input format to one of:"));
+    puts(HELP_TEXT("i FORMAT", "input FORMAT ", "Set the input format to one of:"));
     for(i=0; 1; i++) {
       const char *help_name;
       const char *help_label;
       if(raptor_syntaxes_enumerate(i, &help_name, &help_label, NULL, NULL))
         break;
-      printf("    %-12s            %s", help_name, help_label);
+      printf("    %-14s          %s", help_name, help_label);
+      if(!i)
+        puts(" (default)");
+      else
+        putchar('\n');
+    }
+    puts(HELP_TEXT("o FORMAT", "output FORMAT", "Set the output format to one of:"));
+    for(i=0; 1; i++) {
+      const char *help_name;
+      const char *help_label;
+      if(raptor_serializers_enumerate(i, &help_name, &help_label, NULL, NULL))
+        break;
+      printf("    %-14s          %s", help_name, help_label);
       if(!i)
         puts(" (default)");
       else
         putchar('\n');
     }
-    puts(HELP_TEXT("o FORMAT", "output FORMAT", "Set output format to one of:"));
-    puts("    'simple'                A simple format (default)\n    'ntriples'              N-Triples");
     puts(HELP_TEXT("m MODE", "mode MODE  ", "Set parser mode - 'lax' (default) or 'strict'"));
     puts("\nAdditional options:");
-    puts(HELP_TEXT("c", "count           ", "Count triples - no output"));
+    puts(HELP_TEXT("c", "count           ", "Count triples only - do not print them."));
     puts(HELP_TEXT("e", "ignore-errors   ", "Ignore error messages"));
-    puts(HELP_TEXT("f FEATURE(=VALUE)", "feature FEATURE(=VALUE)", HELP_PAD "Set parser feature - use `help' for a list"));
-    puts(HELP_TEXT("g", "guess           ", "Guess input syntax"));
+    puts(HELP_TEXT("f FEATURE(=VALUE)", "feature FEATURE(=VALUE)", HELP_PAD "Set parser or serializer features" HELP_PAD "Use `-f help' for a list of valid features"));
+    puts(HELP_TEXT("g", "guess           ", "Guess the input syntax"));
     puts(HELP_TEXT("q", "quiet           ", "No extra information messages"));
     puts(HELP_TEXT("r", "replace-newlines", "Replace newlines with spaces in literals"));
     puts(HELP_TEXT("s", "scan            ", "Scan for <rdf:RDF> element in source"));
     puts(HELP_TEXT("w", "ignore-warnings ", "Ignore warning messages"));
     puts(HELP_TEXT("v", "version         ", "Print the Raptor version"));
     puts("\nReport bugs to <redland-dev@lists.librdf.org>.");
-    puts("Raptor home page: http://www.redland.opensource.ac.uk/raptor/");
+    puts("Raptor home page: http://librdf.org/raptor/");
     exit(0);
   }
 
@@ -504,8 +607,14 @@ main(int argc, char *argv[])
   if(scanning)
     raptor_set_feature(rdf_parser, RAPTOR_FEATURE_SCANNING, 1);
 
-  if(feature_value >= 0)
-    raptor_set_feature(rdf_parser, feature, feature_value);
+  if(parser_feature_value >= 0)
+    raptor_set_feature(rdf_parser, 
+                       parser_feature, parser_feature_value);
+  if(parser_feature_string_value)
+    raptor_parser_set_feature_string(rdf_parser, 
+                                     parser_feature,
+                                     parser_feature_string_value);
+
 
   if(!quiet) {
     if (filename) {
@@ -526,6 +635,44 @@ main(int argc, char *argv[])
   raptor_set_statement_handler(rdf_parser, NULL, print_statements);
 
 
+  if(serializer_syntax_name) {    
+    serializer=raptor_new_serializer(serializer_syntax_name);
+    if(!serializer) {
+      fprintf(stderr, 
+              "%s: Failed to create raptor serializer type %s\n", program,
+              serializer_syntax_name);
+      return(1);
+    }
+
+    if(namespace_declarations) {
+      int i;
+      for(i=0; i< raptor_sequence_size(namespace_declarations); i++) {
+        struct namespace_decl *nd=(struct namespace_decl *)raptor_sequence_get_at(namespace_declarations, i);
+        raptor_uri *ns_uri=NULL;
+        if(nd->uri_string)
+          ns_uri=raptor_new_uri(nd->uri_string);
+        
+        raptor_serialize_set_namespace(serializer, ns_uri, nd->prefix);
+        if(ns_uri)
+          raptor_free_uri(ns_uri);
+      }
+      raptor_free_sequence(namespace_declarations);
+      namespace_declarations=NULL;
+    }
+    
+    if(serializer_feature_value >= 0)
+      raptor_serializer_set_feature(serializer, 
+                                    serializer_feature, serializer_feature_value);
+    if(serializer_feature_string_value)
+      raptor_serializer_set_feature_string(serializer, 
+                                           serializer_feature,
+                                           serializer_feature_string_value);
+
+    raptor_serialize_start_to_file_handle(serializer, base_uri, stdout);
+
+  }
+  
+
   /* PARSE the URI as RDF/XML */
   rc=0;
   if(!uri || filename) {
@@ -544,6 +691,12 @@ main(int argc, char *argv[])
 
   raptor_free_parser(rdf_parser);
 
+  if(serializer) {
+    raptor_serialize_end(serializer);
+    raptor_free_serializer(serializer);
+  }
+  
+
   if(!quiet)
     fprintf(stdout, "%s: Parsing returned %d statements\n", program,
             statement_count);
@@ -554,6 +707,9 @@ main(int argc, char *argv[])
   if(free_uri_string)
     raptor_free_memory(uri_string);
 
+  if(namespace_declarations)
+    raptor_free_sequence(namespace_declarations);
+
   raptor_finish();
 
   if(error_count && !ignore_errors)
